<?php

namespace Modules\Small\Http\Controllers\App;


use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Maatwebsite\Excel\Facades\Excel;

use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Contracts\Support\Renderable;
use Modules\Small\Exports\SmallExport;
use Modules\Small\Http\Requests\SmallStoreRequest;
use Modules\Small\Http\Repositories\SmallRepository;


class SmallController extends Controller
{
    private $small_repo;

    public function __construct(SmallRepository $small)
    {
        $this->small_repo = $small;

        // Définition des permissions par méthode
        $this->middleware('permission:module-small-read')->only(['index']);
        $this->middleware('permission:module-small-create')->only(['create', 'duplicate']);
        $this->middleware('permission:module-small-update')->only(['edit']);
        $this->middleware('permission:module-small-delete')->only(['destroy']);

        view()->share('route_prefix', "module.small");
    }

    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index()
    {
        $breadcrumb = array(
            'Accueil'  => route('modulus.dashboard'),
            'Exemples' => null
        );

        return view('small::app.small.index')->with([
            'title'      => '<i class="fa fa-magic mr-1" aria-hidden="true"></i>Exemples',
            'breadcrumb' => $breadcrumb,
        ]);
    }

    /**
     * Show the form for creating a new resource.
     * @return Renderable
     */
    public function create()
    {
        $small = $this->small_repo->new();

        $breadcrumb = array(
            'Accueil'  => route('modulus.dashboard'),
            'Exemples' => route('module.small.index'),
            'Ajouter'  => null
        );

        return view('small::app.small.form')->with([
            'title'           => '<i class="fa fa-magic mr-1" aria-hidden="true"></i>Exemples',
            'action'          => '<i class="fa fa-plus mr-2"></i>Création',
            'breadcrumb'      => $breadcrumb,
            'item'            => $small
        ]);
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function edit($id)
    {
        $small = $this->small_repo->findOrFail($id);


        $breadcrumb = array(
            'Accueil'  => route('modulus.dashboard'),
            'Exemples' => route('module.small.index'),
            'Modifier' => null
        );

        return view('small::app.small.form')->with([
            'title'           => '<i class="fa fa-magic mr-1"></i>Exemples',
            'action'          => '<i class="fa fa-plus mr-2"></i>Modification',
            'breadcrumb'      => $breadcrumb,
            'item'            => $small
        ]);
    }


    /**
     * Store a newly created resource in storage.
     * @param SmallStoreRequest $request
     * @return Renderable
     */
    public function store(SmallStoreRequest $request, $id = null)
    {
        return $this->update($request, $id);
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function show(Request $request, $id)
    {
        $small = $this->small_repo->findOrFail($id);

        $breadcrumb = array(
            'Accueil'  => route('modulus.dashboard'),
            'Exemples' => route('module.small.index'),
            'Voir'     => null
        );

        return view('small::app.small.show')->with([
            'title'           => '<i class="fa fa-magic mr-1"></i>Exemples',
            'subtitle'        => $small->name,
            'breadcrumb'      => $breadcrumb,
            'item'            => $small
        ]);
    }

    /**
     * Update the specified resource in storage.
     * @param SmallStoreRequest $request
     * @param int $id
     * @return Renderable
     */
    public function update(SmallStoreRequest $request, $id)
    {
        $small_data = collect($request->except(['_token']));

        $small = $this->small_repo->store($small_data, $id);

        Session::flash('success', "Exemple ". (is_null($id) ? 'créé' : 'modifié') . " avec succès");

        return Redirect::route('module.small.index');
    }

    public function duplicate($id)
    {
        $this->checkPermissionAccess($id);

        $this->small_repo->duplicate($id);

        Session::flash('success', "Exemple dupliqué avec succès");

        return Redirect::route('module.small.index');
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy($id)
    {
        $this->checkPermissionAccess($id);

        $this->small_repo->destroy($id);

        Session::flash('success', 'Exemple supprimé avec succès');

        return Redirect::route('module.small.index');
    }

    /**
     * Exporte les données de la exemple
     *
     * @param int $id
     * @return void
     */
    public function export()
    {
        return Excel::download(new SmallExport, date('Ymd_His_') . 'export-module-small.xlsx');
    }
}
