<?php

namespace Modules\Small\Http\Repositories;

use Illuminate\Support\Str;
use Illuminate\Support\Collection;
use Modules\Small\Models\Small;
use Goldenscarab\Modulus\App\Http\Repositories\RepositoryInterface;


class SmallRepository implements RepositoryInterface {

	public function all($columns = array('*'))
	{
		return Small::select($columns)->orderBy('id', 'asc')->get();
	}

	public function list($sorting, $direction = 'asc', $perpage = 15, $search = "", $user_id = null)
	{
        // Si tri via relation
        if (Str::contains($sorting, '.')) {

            // Préparation de la jointure
            $relation_filter = explode('.', $sorting);
            $table_join      = $relation_filter[0];
            $primary_key     = $table_join . '.id';
            $foreign_key     = Str::singular(collect(explode('_', $table_join))->last()) . '_id';

            // Récupération des éléments
            $list = Small::select('small.*')
                ->join($table_join, $primary_key, '=', $foreign_key)
                ->user($user_id)
                ->orderBy($sorting, $direction)
                ->search($search)->paginate($perpage);

        } else {
            $list = Small::search($search)->user($user_id)->orderBy($sorting, $direction)->paginate($perpage);
        }
        return $list;
	}

	public function published($columns = array('*'))
	{
		return Small::select($columns)->active()->orderBy('title', 'asc')->get();
	}

	public function new()
	{
		return new Small;
	}

	public function store(Collection $data, $id = null)
	{
		if (is_null($id)) {
			$small = new Small;
		} else {
			$small = Small::findOrFail($id);
		}

		$small->name    = $data->get('name');
		$small->user_id = $data->get('user_id');

        $small->save();

        return $small;
	}

	public function find($id, $columns = array('*'))
    {
        return Small::select($columns)->find($id);
    }

    public function findOrFail($id, $columns = array('*'))
    {
        return Small::select($columns)->findOrFail($id);
    }

    public function findBy($field, $value, $columns = array('*'))
    {
        return Small::select($columns)->where($field, '=', $value)->first();
    }

    public function findOrFailBy($field, $value, $columns = array('*'))
    {
        return Small::select($columns)->where($field, '=', $value)->firstOrFail();
    }

    public function findAllBy($field, $value, $columns = array('*'))
    {
        return Small::select($columns)->where($field, '=', $value)->get();
    }

    public function findWhere($where, $columns = array('*'))
    {
        return Small::select($columns)->whereRaw($where)->get();
    }

    public function search($search)
    {
        return Small::search($search)->get();
    }

    public function destroy($id)
    {
        $item = Small::findOrFail($id);
        return $item->delete();
    }

    public function first()
    {
        return Small::first();
    }

    public function lasts($limit)
    {
        return Small::latest()->limit($limit)->get();
    }

    public function duplicate($id)
    {
        $item = Small::findOrFail($id);
        $clone = $item->replicate();

        $availables = ['title', 'name', 'label', 'slug'];

        foreach ($availables as $available) {
            if (isset($clone->{$available})) {
                $clone->{$available} = $clone->{$available} . ' (copie)';
            }
        }

        $clone->start_at = null;
        $clone->end_at   = null;

        $clone->save();

        return $clone;
    }

    public function truncate()
    {
        return Small::truncate();
    }

    private function calcNextPosition()
    {
        $item = Small::orderBy('position', 'desc')->first();

        if (is_null($item)) {
            return 1;
        }

        return $item->position + 1;
    }

}
