<?php

namespace Goldenscarab\Modulus\App\Http\Controllers\App;

use App\Http\Controllers\Controller;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Goldenscarab\Modulus\App\Http\Exports\TrackingExport;

use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Redirect;
use Goldenscarab\Modulus\App\Http\Repositories\TrackerRepository;
use Modules\Device\Http\Services\Facades\DeviceService;

class DashboardController extends Controller
{

    private $tracker_repo;

    public function __construct(TrackerRepository $tracker)
    {
        $this->tracker_repo = $tracker;
    }


    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $current_week  = $this->tracker_repo->currentWeek();
        $current_month = $this->tracker_repo->currentMonth();
        $last_week     = $this->tracker_repo->lastWeek();
        $last_month    = $this->tracker_repo->lastMonth();
        $current_year  = $this->tracker_repo->currentYear();
        $last_year     = $this->tracker_repo->lastYear();

        // Groupement des visite par mois de l'année courante
        $current_year_by_month = $current_year->groupBy(function ($item, $key) {
            return ucfirst($item->created_at->format('m'));
        });

        // Groupement des visite par mois de l'année courante
        $last_year_by_month = $last_year->groupBy(function ($item, $key) {
            return ucfirst($item->created_at->format('m'));
        });

        return view('modulus::app.dashboard.index')->with([
            'title'                 => "Tableau de bord",
            'subtitle'              => "Analyse",
            'current_week'          => $current_week,
            'current_month'         => $current_month,
            'current_year'          => $current_year,
            'current_year_by_month' => $current_year_by_month,
            'last_week'             => $last_week,
            'last_month'            => $last_month,
            'last_year'             => $last_year,
            'last_year_by_month'    => $last_year_by_month,

        ]);
    }

    /**
     * Export des logs sous forme de tableau Excel
     * @return Download Le fichier Excel d'export
     */
    public function export()
    {
        return Excel::download(new TrackingExport, date('Ymd_His_') . 'export-tracking.xlsx');
    }

    /**
     * Purge le dossier des logs
     * @return Redirect dashbord
     */
    public function truncate()
    {
        $this->tracker_repo->truncate();

        Session::flash('success', "Logs effacés avec suucès");

        return Redirect::route('back.dashboard');
    }

}
