<?php

namespace Goldenscarab\Modulus\App\Http\Exports;


use Goldenscarab\Modulus\App\Http\Repositories\TrackerRepository;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Events\BeforeExport;
use Maatwebsite\Excel\Events\AfterSheet;
use Modules\Setting\Http\Services\Facades\SettingService;



class TrackingExport implements FromCollection, WithHeadings, ShouldAutoSize, WithEvents
{
    private $prop_title;

    private $prop_description;

    private $prop_author;

    private $prop_subject;

    private $prop_company;

    private $sheet_title;

    private $sheet_colums;



	public function __construct()
    {
		$this->prop_description = "Export des tracking de visites sur le site ". SettingService::get('site_name');
		$this->prop_title       = "Tracking visites";
		$this->prop_author      = SettingService::get('meta_author');
		$this->prop_subject     = "Export du site web ". SettingService::get('site_name');
		$this->prop_company     = "Goldenscarab";
		$this->sheet_title      = 'Traking';
		$this->sheet_colums     = array(
            'id'             => 'ID',
            'ip'             => 'IP',
            'connected_user' => 'Visiteur',
            'zip_code'       => 'Code Postal',
            'city'           => 'Ville',
            'country_name'   => 'Pays',
            'latitude'       => 'Latitude',
            'longitude'      => 'Longitude',
            'screen_width'   => 'Largeur écran',
            'screen_height'  => 'Hauteur écran',
            'created_at'     => 'Date'
	    );
    }

    /**
     * Récupération des données depuis la BDD
     * @return Collection La liste des données à exporter
     */
    public function collection()
    {
        $tracker_repo = new TrackerRepository;
        return $tracker_repo->currentYear(array_keys($this->sheet_colums));
    }

    /**
     * Ajout d'une ligne d'entêtes (les titres)
     * @return array Les titres
     */
    public function headings(): array
    {
        return array_values($this->sheet_colums);
    }

    /**
     * Configuration de la feuille d'export
     * @return array La config
     */
    public function registerEvents(): array
    {
        return [
            BeforeExport::class => function(BeforeExport $event) {

		        // Propriétées du tableau
		        $event->writer->getProperties()
		        			  ->setCreator($this->prop_author)
		        			  ->setLastModifiedBy($this->prop_author)
		        			  ->setTitle($this->prop_title)
		        			  ->setSubject($this->prop_subject)
		        			  ->setCompany($this->prop_company)
		        			  ->setDescription($this->prop_description);
		    },
            AfterSheet::class => function(AfterSheet $event) {

            	// Calcul de la zone des titres
            	$title_end = chr(65 + count($this->sheet_colums) - 1);
                $zone_title = sprintf('A1:%s1', $title_end);

                // Style des titres
                $event->sheet->getDelegate()->getStyle($zone_title)->getFont()->setBold(true);

                // Filtre sur les titres
                $event->sheet->setAutoFilter($zone_title);

                // Volets verrouillés
                $event->sheet->freezePane('A2');

                // Titre de l'onglet
                $event->sheet->setTitle($this->sheet_title);
            },
        ];
    }
}
