<?php

namespace Goldenscarab\Modulus\App\Http\Middleware;

use Closure;
use Illuminate\Support\Facades\Auth;
use Goldenscarab\Modulus\App\Http\Services\Compiler\Facade\CompilerService;
use Modules\Setting\Http\Services\Facades\SettingService;

class Compiler
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next, $guard = null)
    {
        $reflash    = $request->session()->get('reflash');

        // Sauvegarde des messages flash
        $availables_flash = ['errors', '_old_input', 'alert-success', 'alert-error', 'success', 'info', 'warning', 'error', 'danger'];
        $messages_flash = [];
        foreach ($availables_flash as $key) {
            if ($request->session()->has($key)) {
                $messages_flash[$key] = $request->session()->get($key);
            }
        }

        if (setting('cache') != 1) {
            CompilerService::clearAll();
        }

        // Création des différents éléments compilés si manquant
        CompilerService::createHeaderIfNotExists();
        CompilerService::createFooterIfNotExists();
        CompilerService::createFrontCSSIfNotExists();
        CompilerService::createFrontJSIfNotExists();
        CompilerService::createBackCSSIfNotExists();
        CompilerService::createBackJSIfNotExists();

        if (count($messages_flash) > 0 && !$reflash) {

            // Préserve les messages flash
            foreach ($messages_flash as $key => $message) {
                $request->session()->flash($key, $message);
            }

            // Evite la récursivité
           $request->session()->flash('reflash', true);
        } else {
           $request->session()->forget('reflash');
        }

        return $next($request);
    }
}
