<?php

namespace Goldenscarab\Modulus\App\Http\Repositories;

use Goldenscarab\Modulus\App\Models\Module;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Cache;


class ModuleRepository implements RepositoryInterface {

	public function all($columns = array('*'))
	{
		return Module::select($columns)->get();
	}

	public function new()
	{
		return new Module;
	}

	public function published($columns = array('*'))
	{
		return false;
	}

	public function list($filter, $sort = 'asc', $per_page = 15, $search = "")
	{
		return Module::search($search)->orderBy($filter, $sort)->paginate($per_page);
	}

	public function allByPosition()
	{
		return Module::orderBy('priority', 'desc')->get();
	}

	public function store(Collection $data, $id = null)
	{
		if (is_null($id)) {
			$config = new Module;
		} else {
			$config = Module::findOrFail($id);
		}

		$config->name     = $data->get('name');
        $config->slug     = $data->get('slug');
        $config->menu     = $data->get('menu');
        $config->priority = $data->get('priority');

        $config->save();

        Cache::forget('module.configs');

        return $config;
	}

	public function find($id, $columns = array('*'))
	{
		return Module::select($columns)->find($id);
	}

	public function findOrFail($id, $columns = array('*'))
	{
		return Module::select($columns)->findOrFail($id);
	}

	public function findBy($field, $value, $columns = array('*'))
	{
		return Module::select($columns)->where($field, '=', $value)->first();
	}

	public function findOrFailBy($field, $value, $columns = array('*'))
	{
		return Module::select($columns)->where($field, '=', $value)->firstOrFail();
	}

	public function findAllBy($field, $value, $columns = array('*'))
	{
		return Module::select($columns)->where($field, '=', $value)->get();
	}

	public function findWhere($where, $columns = array('*'))
	{
		return Module::select($columns)->whereRaw($where)->get();
	}

	public function search($search)
	{
		return Module::search($search)->orderBy('position', 'asc')->get();
	}

	public function destroy($slug)
	{
        $config = Module::where('slug', $slug)->delete();
        Cache::forget('module.configs');

		return false;
	}

	public function first()
	{
		return Module::first();
	}

	public function lasts($limit)
	{
		return Module::latest()->limit($limit)->get();
	}

	public function duplicate($id)
	{
		return false;
	}

	private function calcNextPosition()
	{
		$config = Module::orderBy('position', 'desc')->first();

		if (is_null($config)) {
			return 1;
		}

		return $config->position + 1;
	}

}
