<?php

namespace Goldenscarab\Modulus\App\Http\Repositories;

use Goldenscarab\Modulus\App\Models\Permission;
use Illuminate\Support\Collection;


class PermissionRepository implements RepositoryInterface {

	public function all($columns = array('*'))
	{
		return Permission::select($columns)->get();
	}

	public function new()
	{
		return new Permission;
	}

	public function published($columns = array('*'))
	{
		return false;
	}

	public function list($filter, $sort = 'asc', $per_page = 15, $search = "")
	{
		return Permission::where(function($query) use ($search) {
			if (!empty($search)) {
				$query->where('name', 'LIKE','%' . $search . '%')
					  ->orWhere('display_name', 'LIKE','%' . $search . '%')
					  ->orWhere('description', 'LIKE','%' . $search . '%');
			}
		})->orderBy($filter, $sort)->paginate($per_page);
	}

	public function store(Collection $data, $id = null)
	{
		if (is_null($id)) {
			$permission = new Permission;
		} else {
			$permission = Permission::findOrFail($id);
		}

		$permission->name         = $data->get('name');
		//$permission->guard_name   = $data->get('guard_name');
		$permission->display_name = $data->get('display_name');
		$permission->description  = $data->get('description');
		$permission->save();

        return $permission;
	}

	public function find($id, $columns = array('*'))
	{
		return Permission::select($columns)->find($id);
	}

	public function findOrFail($id, $columns = array('*'))
	{
		return Permission::select($columns)->findOrFail($id);
	}

	public function findBy($field, $value, $columns = array('*'))
	{
		return Permission::select($columns)->where($field, '=', $value)->first();
	}

	public function findOrFailBy($field, $value, $columns = array('*'))
	{
		return Permission::select($columns)->where($field, '=', $value)->firstOrFail();
	}

	public function findAllBy($field, $value, $columns = array('*'))
	{
		return Permission::select($columns)->where($field, '=', $value)->get();
	}

	public function findWhere($where, $columns = array('*'))
	{
		return Permission::select($columns)->whereRaw($where)->get();
	}

	public function destroy($id)
	{
		return Permission::findOrFail($id)->delete();
	}

	public function first()
	{
		return Permission::first();
	}

	public function lasts($limit)
	{
		return Permission::latest()->limit($limit)->get();
	}

	public function search($search)
	{
		return Permission::search($search)->get();
	}

	public function truncate()
	{
		return Permission::truncate();
	}

	public function duplicate($id)
	{
		return false;
	}
}
