<?php

namespace Goldenscarab\Modulus\App\Http\Repositories;

use Spatie\Permission\Models\Role;
use Illuminate\Support\Collection;


class RoleRepository implements RepositoryInterface {

	public function all($columns = array('*'))
	{
		return Role::select($columns)->get();
	}

	public function new()
	{
		return new Role;
	}

	public function published($columns = array('*'))
	{
		return false;
	}

	public function list($filter, $sort = 'asc', $per_page = 15, $search = "")
	{
		return Role::where(function($query) use ($search) {
			if (!empty($search)) {
				$query->where('name', 'LIKE','%' . $search . '%')
					  ->orWhere('display_name', 'LIKE','%' . $search . '%')
					  ->orWhere('description', 'LIKE','%' . $search . '%');
			}
		})->orderBy($filter, $sort)->paginate($per_page);
	}

	public function store(Collection $data, $id)
	{
		if (is_null($id)) {
			$role = new Role;
		} else {
			$role = Role::findOrFail($id);
		}

		$role->name         = $data->get('name');
		//$role->guard_name   = $data->get('guard_name');
		$role->display_name = $data->get('display_name');
		$role->description  = $data->get('description');

		$role->save();

		// Si il y a des permissions à activer
		if ($data->has('permissions')) {

			$role->syncPermissions($data->get('permissions'));
		}

        return $role;
	}

	public function find($id, $columns = array('*'))
	{
		return Role::with('perms')->select($columns)->find($id);
	}

	public function findOrFail($id, $columns = array('*'))
	{
		return Role::select($columns)->findOrFail($id);
	}

	public function findBy($field, $value, $columns = array('*'))
	{
		return Role::with('perms')->select($columns)->where($field, '=', $value)->first();
	}

	public function findOrFailBy($field, $value, $columns = array('*'))
	{
		return Role::select($columns)->where($field, '=', $value)->firstOrFail();
	}

	public function findAllBy($field, $value, $columns = array('*'))
	{
		return Role::with('perms')->select($columns)->where($field, '=', $value)->get();
	}

	public function findWhere($where, $columns = array('*'))
	{
		return Role::with('perms')->select($columns)->whereRaw($where)->get();
	}

	public function search($search)
	{
		return Role::search($search)->get();
	}

	public function destroy($id)
	{
		return Role::findOrFail($id)->delete();
	}

	public function first()
	{
		return Role::first();
	}

	public function lasts($limit)
	{
		return Role::latest()->limit($limit)->get();
	}

	public function truncate()
	{
		return Role::truncate();
	}

	public function duplicate($id)
	{
		return false;
	}

}
