<?php

namespace Goldenscarab\Modulus\App\Http\Repositories;

use Goldenscarab\Modulus\App\Models\Tracker;
use Illuminate\Support\Collection;


class TrackerRepository implements RepositoryInterface {

	public function all($columns = array('*'))
	{
		return Tracker::select($columns)->orderBy('id', 'desc')->get();
	}

	public function list($filter, $sort = 'asc', $per_tracker = 15, $search = "")
	{
		return Tracker::search($search)->orderBy($filter, $sort)->paginate($per_tracker);
	}

	public function published($columns = array('*'))
	{
		return Tracker::select($columns)->active()->orderBy('title', 'asc')->get();
	}

	public function new()
	{
		return new Tracker;
	}

	public function store(Collection $data, $id = null)
	{
		if (is_null($id)) {
			$tracker = new Tracker;
		} else {
			$tracker = Tracker::findOrFail($id);
		}

		$tracker->ip              = $data->get('ip');
		$tracker->connected_user  = $data->get('connected_user');
		$tracker->request_uri     = $data->get('request_uri');
		$tracker->request_method  = $data->get('request_method');
		$tracker->http_user_agent = $data->get('http_user_agent');
		$tracker->provider        = $data->get('provider');
		$tracker->zip_code        = $data->get('zip_code');
		$tracker->city            = $data->get('city');
		$tracker->region_name     = $data->get('region_name');
		$tracker->country_name    = $data->get('country_name');
		$tracker->latitude        = $data->get('latitude');
		$tracker->longitude       = $data->get('longitude');
		$tracker->screen_width    = $data->get('screen_width');
		$tracker->screen_height   = $data->get('screen_height');
        $tracker->save();

        return $tracker;
	}

	public function find($id, $columns = array('*'))
	{
		return Tracker::select($columns)->find($id);
	}

	public function findOrFail($id, $columns = array('*'))
	{
		return Tracker::select($columns)->findOrFail($id);
	}

	public function findBy($field, $value, $columns = array('*'))
	{
		return Tracker::select($columns)->where($field, '=', $value)->first();
	}

	public function findOrFailBy($field, $value, $columns = array('*'))
	{
		return Tracker::select($columns)->where($field, '=', $value)->firstOrFail();
	}

	public function findAllBy($field, $value, $columns = array('*'))
	{
		return Tracker::select($columns)->where($field, '=', $value)->get();
	}

	public function findWhere($where, $columns = array('*'))
	{
		return Tracker::select($columns)->whereRaw($where)->get();
	}

	public function search($search)
	{
		return Tracker::search($search)->orderBy('lastname', 'asc')->get();
	}

	public function destroy($id)
	{
		return Tracker::findOrFail($id)->delete();
	}

	public function first()
	{
		return Tracker::first();
	}

	public function lasts($limit)
	{
		return Tracker::latest()->limit($limit)->get();
	}

	public function truncate()
	{
		return Tracker::truncate();
	}

	public function currentWeek($columns = array('*'))
	{
		return Tracker::select($columns)->currentWeek()->orderBy('id', 'desc')->get();
	}

	public function lastWeek($columns = array('*'))
	{
		return Tracker::select($columns)->lastWeek()->orderBy('id', 'desc')->get();
	}

	public function currentMonth($columns = array('*'))
	{
		return Tracker::select($columns)->currentMonth()->orderBy('id', 'asc')->get();
	}

	public function lastMonth($columns = array('*'))
	{
		return Tracker::select($columns)->lastMonth()->orderBy('id', 'asc')->get();
	}

	public function currentYear($columns = array('*'))
	{
		return Tracker::select($columns)->currentMonth()->orderBy('id', 'asc')->get();
	}

	public function lastYear($columns = array('*'))
	{
		return Tracker::select($columns)->lastMonth()->orderBy('id', 'asc')->get();
	}

	public function duplicate($id)
	{
		return false;
	}
}
