<?php

namespace Goldenscarab\Modulus\App\Http\Repositories;

use Goldenscarab\Modulus\App\Models\User;
use Illuminate\Support\Str;
use Illuminate\Support\Collection;


class UserRepository implements RepositoryInterface {

	public function all($columns = array('*'))
	{
		return User::with('roles')->select($columns)->orderBy('lastname', 'asc')->get();
	}

	public function list($filter, $sort = 'asc', $per_page = 15, $search = "", $role_id = null)
	{
		if ($role_id) {

			return User::whereHas('roles', function($query) use ($role_id) {
				$query->where('role_id', $role_id);
			})->search($search)->orderBy($filter, $sort)->paginate($per_page);
		}

		return User::with('roles')->search($search)->orderBy($filter, $sort)->paginate($per_page);
	}

	public function published($columns = array('*'))
	{
		return User::active()->select($columns)->orderBy('lastname', 'asc')->get();
	}

	public function new()
	{
		return new User;
	}

	public function store(Collection $data, $id)
	{
		if (is_null($id)) {
            $user = new User;
		} else {
			$user = User::findOrFail($id);
		}
        $user->api_token = Str::random(60);
		$user->avatar    = $data->get('avatar');
		$user->lastname  = $data->get('lastname');
		$user->firstname = $data->get('firstname');
		$user->email     = $data->get('email');
		$user->comment   = $data->get('comment');
		$user->company   = $data->get('company');
		$user->activity  = $data->get('activity');
		$user->telephone = $data->get('telephone');
		$user->address   = $data->get('address');
		$user->zipcode   = $data->get('zipcode');
		$user->city      = $data->get('city');

		// Si le champs active est donné
		if ($data->has('active')) {
			$user->active    = $data->get('active');
		}

		// Si le mot de passe est changé
		if (!is_null($data->get('password'))) {
            $user->password  = bcrypt($data->get('password'));
        }

        $user->save();

        // Changement du role de l'utilisateur
        if ($data->has('role')) {
        	$user->syncRoles($data->get('role'));
        }

        return $user;
	}

	public function find($id, $columns = array('*'))
	{
		return User::with('roles')->select($columns)->find($id);
	}

	public function findOrFail($id, $columns = array('*'))
	{
		return User::select($columns)->findOrFail($id);
	}

	public function findBy($field, $value, $columns = array('*'))
	{
		return User::with('roles')->select($columns)->where($field, '=', $value)->first();
	}

	public function findOrFailBy($field, $value, $columns = array('*'))
	{
		return User::select($columns)->where($field, '=', $value)->firstOrFail();
	}

	public function findAllBy($field, $value, $columns = array('*'))
	{
		return User::with('roles')->select($columns)->where($field, '=', $value)->get();
	}

	public function findWhere($where, $columns = array('*'))
	{
		return User::with('roles')->select($columns)->whereRaw($where)->get();
	}

	public function search($search)
	{
		return User::search($search)->orderBy('lastname', 'asc')->get();
	}

	public function destroy($id)
	{
		return User::findOrFail($id)->delete();
	}

	public function first()
	{
		return User::first();
	}

	public function lasts($limit)
	{
		return User::latest()->limit($limit)->get();
	}

	public function truncate()
	{
		return User::truncate();
	}

	public function duplicate($id)
	{
		return false;
	}

	public function getAvatars()
	{
		return User::getAvatars();
	}

}
