<?php

namespace Goldenscarab\Modulus\App\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;
use Goldenscarab\Modulus\App\Rules\Telephone;

class ProfileStoreRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {

        return [
            'lastname'  => 'required|max:100|string',
            'firstname' => 'required|max:100|string',
            'email'     => 'required|email|unique:users,email,' . auth()->user()->id,

            'company'   => 'nullable|max:100|string',
            'activity'  => 'nullable|max:100|string',
            'telephone' => ['nullable', 'max:100', new Telephone],

            'address'   => 'nullable|max:254|string',
            'zipcode'   => 'nullable|max:99999|integer',
            'city'      => 'nullable|max:100|string',
            'rgpd'      => 'required|in:1'
        ];
    }


    /**
     * Get custom attributes for validator errors.
     *
     * @return array
     */
    public function attributes()
    {
        return [
            'lastname'  => 'Nom',
            'firstname' => 'Prénom',
            'email'     => 'Email',

            'company'   => 'Société',
            'activity'  => 'Activité',
            'telephone' => 'Téléphone',

            'address'   => 'Adresse',
            'zipcode'   => 'Code postal',
            'city'      => 'Ville',
            'rgpd'      => 'Règlement Général sur la Protection des Données'
        ];
    }
}
