<?php

namespace Goldenscarab\Modulus\App\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;

class UserStoreRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        // Vérification des droits de création et modification
        if (empty($this->route('id'))) {

            return auth()->user()->can('user-create');

        } else if (auth()->user()->can('user-update')) {

            return true;

        } else {

            // Autorisé uniquement à son propre profil
            return (auth()->id == $this->route('id'));
        }
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {

        return [
            'avatar'    => 'string',
            'lastname'  => 'required|min:3|max:100|string',
            'firstname' => 'required|min:3|max:100|string',
            'email'     => 'required|email|unique:users,email,' . $this->route('id'),
            'role'      => 'array',
            'comment'   => 'max:250',
            'password'  => 'required_without:id|between:5,56|nullable',
            'active'    => 'required_with:role|in:0,1'
        ];
    }


    /**
     * Get custom attributes for validator errors.
     *
     * @return array
     */
    public function attributes()
    {
        return [
            'avatar'    => 'Avatar',
            'lastname'  => 'Nom',
            'firstname' => 'Prénom',
            'comment'   => 'Commentaire',
            'role'      => 'Role',
            'password'  => 'Mot de passe',
            'active'    => 'Actif'
        ];
    }
}
