<?php

namespace Goldenscarab\Modulus\App\Http\Services\Compiler;

use Illuminate\Support\Str;
use Illuminate\Support\Facades\File;
use Goldenscarab\Modulus\App\Http\Repositories\CompilerRepository;
use Goldenscarab\Modulus\App\Http\Services\Module\Facade\ModuleService;
use Modules\Menu\Http\Services\Facades\MenuService;
use Modules\Page\Http\Services\Facades\PageService;
use Modules\Custom\Http\Services\Facades\CustomService;


class CompilerService
{

	private $front_css_file_name = 'front_compiled.css';
	private $front_js_file_name  = 'front_compiled.js';
	private $back_css_file_name  = 'back_compiled.css';
	private $back_js_file_name   = 'back_compiled.js';
	private $header_file_name    = '_header.blade.php';
	private $footer_file_name    = '_footer.blade.php';

    private $compiled_path = null;

    public function __construct()
    {
        $this->compiled_path = resource_path('views') . '/' . config('modulus.compiled_folder', 'n/a');
    }

    /* Views */
    public function hasView($file_name)
    {
    	$path_file = $this->compiled_path . $file_name;
    	return File::exists($path_file);
    }

    public function putView($file_name, $content)
    {
    	$path_file = $this->compiled_path . $file_name;
    	File::put($path_file, $content);
    }

    public function deleteView($file_name)
    {
    	$path_file = $this->compiled_path . $file_name;
    	File::delete($path_file);
    }

    public function deleteViews()
    {
    	$path = $this->compiled_path;
    	File::cleanDirectory($path);
    }

    public function createViewIfNotExists($uri)
    {
    	$view_name = Str::slug($uri) . '.blade.php';

        // Si le dossier de travail n'existe pas
        if (!File::isDirectory($this->compiled_path)) {
            File::makeDirectory($this->compiled_path, 0777, true, true);
        }

        // Création de la vue compilée
        if (!$this->hasView($view_name)) {
            $compiled = PageService::makeCompiledContent($uri);
            $this->putView($view_name, $compiled);
        }
    }

    /* Header */
    public function hasHeader()
    {
    	$path_file = resource_path('views') . '/' . config('modulus.compiled_folder') . $this->header_file_name;
    	return File::exists($path_file);
    }
    public function putHeader($content)
    {
    	$path_file = resource_path('views') . '/' . config('modulus.compiled_folder') . $this->header_file_name;
    	File::put($path_file, $content);
    }
    public function deleteHeader()
    {
    	$path_file = resource_path('views') . '/' . config('modulus.compiled_folder') . $this->header_file_name;
    	File::delete($path_file);
    }
    public function createHeaderIfNotExists()
    {
    	if (!$this->hasHeader()) {
            $this->putHeader(CustomService::makeCompiledHeader());
        }
    }


    /* Footer */
    public function hasFooter()
    {
    	$path_file = resource_path('views') . '/' . config('modulus.compiled_folder') . $this->footer_file_name;
    	return File::exists($path_file);
    }
    public function putFooter($content)
    {
    	$path_file = resource_path('views') . '/' . config('modulus.compiled_folder') . $this->footer_file_name;
    	File::put($path_file, $content);
    }
    public function deleteFooter()
    {
    	$path_file = resource_path('views') . '/' . config('modulus.compiled_folder') . $this->footer_file_name;
        File::delete($path_file);
    }
    public function createFooterIfNotExists()
    {
    	if (!$this->hasFooter()) {
            $this->putFooter(CustomService::makeCompiledFooter());
        }
    }


    /* CSS */
    public function hasFrontCSS()
    {
    	$path_file = public_path('css') . '/' . $this->front_css_file_name;
    	return File::exists($path_file);
    }
    public function putFrontCSS($content)
    {
    	$path_file = public_path('css') . '/' . $this->front_css_file_name;
    	File::put($path_file, $content);
    }
    public function deleteFrontCSS()
    {
    	$path_file = public_path('css') . '/' . $this->front_css_file_name;
    	File::delete($path_file);
    }
    public function createFrontCSSIfNotExists()
    {
    	if (!$this->hasFrontCSS()) {
            $css = '';
            $css .= CustomService::makeCompiledCSS();
            $css .= PageService::makeCompiledCSS();
            $css .= MenuService::makeCompiledCSS();
            $css .= ModuleService::makeCompiledCSSFront();
            $this->putFrontCSS($css);
        }
    }

    public function hasBackCSS()
    {
    	$path_file = public_path('css') . '/' . $this->back_css_file_name;
    	return File::exists($path_file);
    }
    public function putBackCSS($content)
    {
    	$path_file = public_path('css') . '/' . $this->back_css_file_name;
    	File::put($path_file, $content);
    }
    public function deleteBackCSS()
    {
    	$path_file = public_path('css') . '/' . $this->back_css_file_name;
    	File::delete($path_file);
    }
    public function createBackCSSIfNotExists()
    {
    	if (!$this->hasBackCSS()) {
            $css = '';
            $css .= ModuleService::makeCompiledCSSBack();
            $this->putBackCSS($css);
        }
    }

    /* JS */
    public function hasFrontJS()
    {
    	$path_file = public_path('js') . '/' . $this->front_js_file_name;
    	return File::exists($path_file);
    }
    public function putFrontJS($content)
    {
    	$path_file = public_path('js') . '/' . $this->front_js_file_name;
    	File::put($path_file, $content);
    }
    public function deleteFrontJS()
    {
    	$path_file = public_path('js') . '/' . $this->front_js_file_name;
    	File::delete($path_file);
    }
    public function createFrontJSIfNotExists()
    {
    	if (!$this->hasFrontJS()) {
            $js = '';
            $js .= CustomService::makeCompiledJS();
            $js .= PageService::makeCompiledJS();
            $js .= MenuService::makeCompiledJS();
            $js .= ModuleService::makeCompiledJSFront();
            $this->putFrontJS($js);
        }
    }
    public function hasBackJS()
    {
    	$path_file = public_path('js') . '/' . $this->back_js_file_name;
    	return File::exists($path_file);
    }
    public function putBackJS($content)
    {
    	$path_file = public_path('js') . '/' . $this->back_js_file_name;
    	File::put($path_file, $content);
    }
    public function deleteBackJS()
    {
    	$path_file = public_path('js') . '/' . $this->back_js_file_name;
    	File::delete($path_file);
    }
    public function createBackJSIfNotExists()
    {
    	if (!$this->hasBackJS()) {
            $js = '';
            $js .= ModuleService::makeCompiledJSBack();
            $this->putBackJS($js);
        }
    }

    public function clearAll()
    {
        $this->deleteViews();
        $this->deleteHeader();
        $this->deleteFooter();
        $this->deleteFrontCSS();
        $this->deleteFrontJS();
        $this->deleteBackCSS();
        $this->deleteBackJS();
    }

}
