<?php

namespace Goldenscarab\Modulus\App\Http\Services\Log;

use Illuminate\Support\Str;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;

class LogService
{
    private $base_path = 'logs/';

    private $expire_days = 100;

    public function writeAsText($folder, $file_name, String $string)
    {
        $content = $this->makeHeader();
        $content .= $string;

        $this->storeLog($folder, $string);
    }

    public function writeAsArray($folder, $data)
   	{
        $content = $this->makeHeader();
        $content .= print_r($data, true);

        $this->storeLog($folder, $content);
   	}

    public function writeAsJson($folder, $data)
   	{
        $content = $this->makeHeader();
        $content .= json_encode($data, JSON_PRETTY_PRINT) . PHP_EOL;

        $this->storeLog($folder, $content);
   	}

    private function makePath($folder)
    {
        return $this->base_path . $folder . "/";
    }

    private function makeHeader()
    {
        $datetimestamp = \Carbon\Carbon::now()->setTimezone('Europe/Paris')->format('d-m-Y H:i:s');
        $header  = "===============================================================================" . PHP_EOL;
   		$header .=  "                            " . $datetimestamp . PHP_EOL;
        $header .= "===============================================================================" . PHP_EOL;

        return $header;
    }

    private function checkDir($folder)
    {
        $dir = storage_path('app/' . $this->makePath($folder));

        if (!File::isWritable($dir)) {
            File::makeDirectory($dir,  0777, true);
        }
    }

    private function storeLog($folder, $data)
    {
        $file = date('Ymd') . '_' . Str::slug($folder) . '.log';
        $path_name = $this->makePath($folder) . $file;
        $this->checkDir($folder);
        Storage::append($path_name, $data);
   		$this->cleanExpiredLogs($folder);
    }

    private function cleanExpiredLogs($folder)
   	{
   		$absolute_path = storage_path('app/' . $this->makePath($folder));
   		$command = "find " . $absolute_path . "/*.log -mtime +" . $this->expire_days . " -exec rm {} \; ";
   		@exec($command);
    }
}
