<?php

namespace Goldenscarab\Modulus\App\Http\Services\Parser\Workers;

use Symfony\Component\Debug\Exception\FatalThrowableError;


class ParserFactory
{
	/**
	 * Délimiter du tag
	 * @var string
	 */
	private static $tag_delimiter = '|';


	/**
	 * Création du rendu associé à un tag
	 *   1 - Séparation des différentes partie composant le tag -> [ worker.job.method.parameters ]
	 *   2 - Worker : il s'agit de l'ouvrier qui saura interpréter le restant du tag
	 *   3 - Job : il s'agit du métier concerné
	 *   4 - Method : il s'agit de la vue à utiliser dans le métier
	 *   4 - Param : (Optionnelle), Il s'agit des paramètres à utiliser pour le traitement
	 * @param  Array $missions 	Liste des tags à traiter
	 * @return Array       		Tableau associatif : [ tag => rendu ]
	 */
	public static function make($missions)
	{

		// Avant tout on controle que l'on a bien un tableau
		if (is_array($missions)) {

			$rending = [];

			// Parcours de chaque mission (tags sans les [])
			foreach ($missions[1] as $key => $mission) {

				$render = "";

				// On étudie la demande
				$mission_details = explode(self::$tag_delimiter, $mission);

				// Dispatch des données composant la demande
				$tag_worker = array_shift($mission_details);
				$tag_job    = array_shift($mission_details);
				$tag_method = array_shift($mission_details);
				$tag_params = $mission_details;

				// Construction du nom de l'ouvrier compétent pour traiter la demande
				$worker_name =  __NAMESPACE__ . "\\Worker" . ucfirst($tag_worker);

				// On vérifie si l'ouvrier est disponible
				if (class_exists($worker_name)) {

					// Convoquation de l'ouvrier
					$worker = new $worker_name();

					// On lui donne son ordre de mission (objectif)
					$worker->setJob($tag_job);

					// On lui donne la methode de travail
					$worker->setMethod($tag_method);

					// Aller au boulot maintenant, tic tac, tic tac !!!
					// https://www.youtube.com/watch?v=A70xUvyPA_c
					$worker->make($tag_params);

					// On récupère le travail effectué
					$render = $worker->getRender();

					// Création d'une étiquette
					$label = "<!-- # ". $mission . " # -->\n";

					// Ajout de l'étiquette au travail
					$render = $label . $render;

				} else {

					// Ohhh No !!! Il n'y a personne qui sait faire cette demande
					throw new \InvalidArgumentException(":'( - The [$worker_name] worker was not found !");
				}

				// Classement du travail effectué et on passe à la suite
				$rending[$missions[0][$key]] = $render;
			}

		}

		return $rending;
	}

}
