<?php

namespace Goldenscarab\Modulus\App\Http\Services\Permission;

use Illuminate\Support\Str;
use Goldenscarab\Modulus\App\Http\Repositories\RoleRepository;
use Goldenscarab\Modulus\App\Http\Repositories\PermissionRepository;


class PermissionService
{

	private $perm_repo;
	private $role_repo;

	private $types = array(
			'read'   => 'Lire',
			'create' => 'Ajouter',
			'update' => 'Modifier',
			'delete' => 'Supprimer'
		);

	public function __construct(PermissionRepository $permission)
	{
		$this->perm_repo = $permission;
		$this->role_repo = new RoleRepository;
	}

	public function addPermissionsGroup($display_name)
	{
		$name = Str::slug($display_name);

		// On vérifie que la permission n'existe pas déja
		$perm = $this->perm_repo->findBy('name', $name . '-read');

		if($perm) {
			return false;
		}

		// On récupère le role admin pour associer les nouvelles permission
		$role = $this->role_repo->findOrFailBy('name', 'admin');

		// Pour chaque élements du groupe
		foreach ($this->types as $type => $human_type) {
			$data = collect([
				'name'         => $name . '-' . $type,
				'display_name' => $display_name . ' ' . ucfirst($type),
				'description'  => $human_type . ' ' . $display_name
			]);

			// Création de la permission
			$perm = $this->perm_repo->store($data);

			// Ajout de la permission au role admin
			if ($perm) {
				$role->givePermissionTo($perm);
			}

		}

		return true;
	}

	public function deletePermissionsGroup($display_name)
	{
        // On récupère le role admin pour supprimer les permissions
		$role_admin = $this->role_repo->findOrFailBy('name', 'admin');
		//$role_edit  = $this->role_repo->findOrFailBy('name', 'editeur');

		// Pour chaque élements du groupe
		foreach ($this->types as $type => $human_type) {

			// Construction du nom de la permission
			$name = Str::slug($display_name) . '-' . $type;
			// Récupération de la permission
			$perm = $this->perm_repo->findBy('name', $name);

			// Si la permission est trouvée, alors on la supprime
			if($perm) {
				$role_admin->revokePermissionTo($perm);
				$perm->delete();
			}

		}

		return true;
	}

}
