<?php

namespace Goldenscarab\Modulus\App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;

class RequestPostTracker implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    /**
     * Url du web service de Geolocalisation
     * @var string
     */
    private $template_url = "http://api.ipstack.com/%s?access_key=9eeb980eb6e32fdd5b3c9a72e7cc2e9b&format=1";


    private $request;


    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($request)
    {
        $this->request = $request;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {

        $ip = \Request::ip();
        $connected_user = !empty(\Auth::user()) ? \Auth::user()->email : null;

        // On récupère la variable d'environnement serveur
        $server = collect(\Request::server());

        // Construction des données de log
        $tracker = array(
            'ip'              => $ip,
            'connected_user'  => $connected_user,
            'request_uri'     => $server->get('REQUEST_URI'),
            'origin_url'      => $server->get('HTTP_REFERER'),
            'request_method'  => $server->get('REQUEST_METHOD'),
            'http_user_agent' => $server->get('HTTP_USER_AGENT'),
        );

        // Récupération des données geographique de l'ip
        $url = sprintf($this->template_url, $ip);
        $geoip  = collect(@json_decode(@file_get_contents($url)));

        if (!$geoip->has('error')) {

            $tracker['zip']          = $geoip->get('zip');
            $tracker['city']         = $geoip->get('city');
            $tracker['region_name']  = $geoip->get('region_name');
            $tracker['country_name'] = $geoip->get('country_name');
            $tracker['latitude']     = $geoip->get('latitude');
            $tracker['longitude']    = $geoip->get('longitude');
        } else {
            $tracker['error'] = $geoip->get('error');
        }

        // On sauvegarde les données dans la donnée
        $this->request->geoloc = $tracker;
        $this->request->save();
    }
}
