<?php

namespace Goldenscarab\Modulus\App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Support\Facades\Auth;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Request;
use Illuminate\Queue\InteractsWithQueue;

use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Goldenscarab\Modulus\App\Http\Repositories\TrackerRepository;



class Tracker implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    /**
     * Url du web service de Geolocalisation
     * @var string
     */
    private $template_url = "http://api.ipstack.com/%s?access_key=9eeb980eb6e32fdd5b3c9a72e7cc2e9b&format=1";

    /**
     * Liste des utilisateurs à ignorer
     * @var array
     */
    private $ignore_role = ['admin', 'editeur'];


    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $connected_user = Auth::user();

        // Role à ignorer
        if (Auth::check() && $connected_user->hasAnyRole($this->ignore_role)) {
            return false;
        }

        // Données de connexion
        $tracker = array(
            'ip'              => Request::ip(),
            'connected_user'  => !empty($connected_user) ? $connected_user->email : null,
            'request_uri'     => Request::server('REQUEST_URI'),
            'origin_url'      => Request::server('HTTP_REFERER'),
            'request_method'  => Request::server('REQUEST_METHOD'),
            'http_user_agent' => Request::server('HTTP_USER_AGENT'),
            'screen_width'    => $_COOKIE['sw'] ?? null,
            'screen_height'   => $_COOKIE['sh'] ?? null
        );

        // Récupération des données geographique de l'ip
        $url = sprintf($this->template_url, Request::ip());
        $geoip  = collect(@json_decode(@file_get_contents($url)));

        if (!$geoip->has('error')) {

            $tracker['zip_code']     = $geoip->get('zip');
            $tracker['city']         = $geoip->get('city');
            $tracker['region_name']  = $geoip->get('region_name');
            $tracker['country_name'] = $geoip->get('country_name');
            $tracker['latitude']     = $geoip->get('latitude');
            $tracker['longitude']    = $geoip->get('longitude');
        }

        // Sauvegarde en BDD
        $traker_repo = new TrackerRepository;
        $traker_repo->store(collect($tracker));
    }
}
