<?php

namespace Goldenscarab\Modulus\App\Models;

use Spatie\Permission\Traits\HasRoles;
use Illuminate\Notifications\Notifiable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Contracts\Auth\MustVerifyEmail;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;

class User extends Authenticatable
{
    use HasFactory, Notifiable, HasRoles;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name',
        'email',
        'password',
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password',
        'remember_token',
        'api_token'
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    private static $avatars = [
        [
            'label' =>'Homme',
            'image' => '/images/modulus/man.svg'
        ],[
            'label' =>'Femme',
            'image' => '/images/modulus/woman.svg'
        ],
    ];

    protected $appends = ['name'];

    public static function getAvatars()
    {
        return self::$avatars;
    }

    protected function getRoleHtmlAttribute()
    {
        $roles = $this->getRoleNames();

        $html = '';

        foreach ($roles as $role) {
           switch ($role) {
               case 'admin':
                   $html .= '<span class="badge badge-danger mr-1">Admin</span>' ;
                   break;
                case 'editeur':
                   $html .= '<span class="badge badge-warning mr-1">Editeur</span>' ;
                   break;
               default:
                   $html .= '<span class="badge badge-secondary mr-1">' . ucfirst($role) . '</span>' ;
                   break;
           }
        }

        return $html;
    }

    protected function getCreatedAtHtmlAttribute()
    {
        return $this->created_at->format('d/m/Y H:i:s');
    }

    protected function getActiveHtmlAttribute()
    {
        if ($this->attributes['active'] == 1) {
            return '<span class="badge badge-success">Actif<span>';
        } else {
            return '<span class="badge badge-default">Inactif<span>';
        }
    }

    protected function getNameAttribute()
    {
        return $this->attributes['firstname'] . ' ' .  $this->attributes['lastname'];
    }

    public function scopeSearch(Builder $query, $search)
    {
        return $query->where('lastname', 'LIKE','%' . $search . '%')
            ->orWhere('firstname', 'LIKE','%' . $search . '%')
            ->orWhere('email', 'LIKE','%' . $search . '%')
            ->orWhere('comment', 'LIKE','%' . $search . '%');
    }

    public function isActive()
    {
        return $this->attributes['active'] == 1;
    }

    public function isValidated()
    {
        return $this->attributes['validated'] == 1;
    }

    public function scopeActive($request)
    {
        return $request->where('active', 1);
    }
}
