<?php

namespace Goldenscarab\Modulus\App\Providers;

use Carbon\Carbon;

use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Facades\View;
use Illuminate\Foundation\AliasLoader;
use Illuminate\Support\ServiceProvider;
use Goldenscarab\Modulus\App\Http\Services\Module\Facade\ModuleService;

class ModulusServiceProvider extends ServiceProvider
{
    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        // Si execution console alors on quitte
        if (App::runningInConsole()) {
            return true;
        }

        $modulus_view = trim(config('url_prefix'), '/') . '*';

        // Configuration de Carbon dans la langue locale
        Carbon::setLocale(config('app.locale'));
        date_default_timezone_set('Europe/Paris');

        View::composer($modulus_view, function($view){
            $view->with('modules_menu', ModuleService::getModulesConfig());
            //$view->with('route_param', \Request::getQueryString());
        });

    }

    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        // Déclaration des services
        $this->app->bind('AppService', 'Goldenscarab\Modulus\App\Http\Services\App\AppService');
        $this->app->bind('LogService', 'Goldenscarab\Modulus\App\Http\Services\Log\LogService');
        $this->app->bind('UserService', 'Goldenscarab\Modulus\App\Http\Services\User\UserService');
        $this->app->bind('ModuleService', 'Goldenscarab\Modulus\App\Http\Services\Module\ModuleService');
        $this->app->bind('PermissionService', 'Goldenscarab\Modulus\App\Http\Services\Permission\PermissionService');

        // Déclaration des facades des services
        $loader = AliasLoader::getInstance();
        $loader->alias('AppService', 'Goldenscarab\Modulus\App\Http\Services\App\Facade\AppService');
        $loader->alias('LogService', 'Goldenscarab\Modulus\App\Http\Services\Log\Facade\LogService');
        $loader->alias('UserService', 'Goldenscarab\Modulus\App\Http\Services\User\Facade\UserService');
        $loader->alias('ModuleService', 'Goldenscarab\Modulus\App\Http\Services\Module\Facade\ModuleService');
        $loader->alias('PermissionService', 'Goldenscarab\Modulus\App\Http\Services\Permission\Facade\PermissionService');
    }
}
