<?php

namespace Database\Seeders;

use Goldenscarab\Modulus\App\Models\User;
use Goldenscarab\Modulus\App\Models\Permission;
use Spatie\Permission\Models\Role;

use Illuminate\Database\Seeder;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class PermissionsTablesSeeder extends Seeder
{
    /**
     * Run the database seeds.
     *
     * php artisan db:seed --class=PermissionsTablesSeeder
     *
     * @return void
     */
    public function run()
    {
        Schema::disableForeignKeyConstraints();

        DB::table('model_has_roles')->truncate();
        DB::table('model_has_permissions')->truncate();
        DB::table('role_has_permissions')->truncate();
        DB::table('roles')->truncate();
        DB::table('permissions')->truncate();

        Schema::enableForeignKeyConstraints();

        // Suppression du cache
        app()[\Spatie\Permission\PermissionRegistrar::class]->forgetCachedPermissions();

        // Liste des permissions à créer
        $permissions = array(
            'user',
            'role',
            'permission',
            'module'
        );

        // Création des permissions
        foreach ($permissions as $permission) {
			$perm               = new Permission;
			$perm->name         = $permission . '-read';
			$perm->display_name = ucfirst($permission) . ' Read';
			$perm->description  = 'Voir';
			$perm->save();

			$perm               = new Permission;
			$perm->name         = $permission . '-create';
			$perm->display_name = ucfirst($permission) . ' Create';
			$perm->description  = 'Créer';
			$perm->save();

			$perm               = new Permission;
			$perm->name         = $permission . '-update';
			$perm->display_name = ucfirst($permission) . ' update';
			$perm->description  = 'Modifier';
			$perm->save();

			$perm               = new Permission;
			$perm->name         = $permission . '-delete';
			$perm->display_name = ucfirst($permission) . ' Delete';
			$perm->description  = 'Supprimer';
			$perm->save();
        }

        // Création des rôles et association des permissions
        $role               = new Role;
		$role->name         = 'admin';
		$role->display_name = 'Admin';
		$role->description  = 'Super utilisateur';
		$role->save();
		$role->givePermissionTo(Permission::all());

		$role               = new Role;
		$role->name         = 'client';
		$role->display_name = 'Client';
		$role->description  = 'Utilisateur classique';
		$role->save();
		$role->givePermissionTo([]);

        $role               = new Role;
		$role->name         = 'visitor';
		$role->display_name = 'Visiteur';
		$role->description  = 'Utilisateur visiteur';
		$role->save();
		$role->givePermissionTo([]);

		$user = User::findOrfail(1);
		$user->assignRole(['admin']);
    }
}
