
/**
 * Objet principal de l'application
 *
 */
var Modulus = {
    current_page: null,
    current_domain: null,

    initAll: function() {

        this.clipboard.init('.clipboard-copy');
        this.popup.init();
        this.postAjax.init();
        //this.position.init();

        // Init des popovers d'aide
        $('.popup-help').popover({ html: true });

        // Combobox
        $('.select2').select2({
            language: 'fr',
            theme: "bootstrap4",
        });

        // Selecteur de date
        // $('.datepicker').datepicker({
        //     language: 'fr',
        //     format: 'dd/mm/yyyy',
        // });

        // Date picker
        $('.airdatepicker').datepicker({
            language: 'fr',
        });

        // Date time picker
        $('.airdatetimepicker').datepicker({
            language: 'fr',
            timepicker: true,
        });

        $('.airtimepicker').datepicker({
            language: 'fr',
            dateFormat: ' ',
            timepicker: true,
            classes: 'airtimepicker'
        });

        // Editeur de code
        $('.code-editor').codeeditor();

        // Gestionnaire de fichier
        $('.popupfinder').popupfinder();

        // Editeur de texte Wysiwyg
        $('.text-editor').texteditor();

        // Composeur de contenu HTML
        $('.composer-bt4').composerBt4();

        $(document).on('change', '.money', function() {
            var value = $(this).val();
            var price;

            if(value && !value.match(/^\d{1,}[,]\d{2}$/)) {
                price = numeral(value.replace('.', ','));
                $(this).val(price.format('0,0.00'));
            } else if (!value) {
                $(this).val(numeral(0).format('0,0.00'));
            }
        });

        $('.money').trigger('change');
    }
};

Modulus.postAjax = {
    init: function() {

        this.setListeners();
    },

    setListeners: function() {

        $('form.ajax').on('submit', function(event) {

            // Suppression du comportement normal de submit
            event.preventDefault();

            // Lancement du post
            Modulus.postAjax.post(this)

            // On efface les messages au bout de 5 secondes
            window.setTimeout(function () {
                $(".alert.ajax").fadeOut(3000);
            }, 5000);
        });
    },

    post: function(form) {

        // Envoi via ajax
        $.ajax({
            url        : form.action,
            type       : form.method,
            headers    : { 'X-CSRF-Token' : window.Laravel.csrfToken },
            contentType: false,
            processData: false,
            async      : true,
            dataType   : 'json',
            data       : new FormData(form),
            success    : function(result){ Modulus.popup.alert('success', result); },
            error      : function(result){ Modulus.postAjax.error(result) },
        });
    },

    error: function(result) {

        // Cas d'une demande de redirection
        if(result.status === 302) {
            document.location.href = result.responseJSON.redirect
            return true;
        }

        // Si l'erreur de validation
        if(result.status === 422) {

            // Affichage des erreurs
            Modulus.postAjax.displayErrors(result.responseJSON.errors);
            return true;
        }

       Modulus.popup.alert('success', result);
        return false;
    },

    displayErrors: function(errors) {

        // Nettoyage des messages d'erreurs
        Modulus.postAjax.cleanErrors();

        // Construction du message
        var html = "Il y a des erreurs dans la saisie de certains champs :";

        html += "<ul>";

        $.each(errors, function (field, message) {
            html += "<li><b>" + field + "</b> : " + message + "</li>";

            // Mise du champs en mode erreur et un message
            var $field = $('[name="'+ field +'"').parents('.form-group');
            $field.addClass('has-error');
            $field.append('<span class="help-block"><strong>' + message + '</strong></span>');

        });

        html += "</ul>";

        // Ajout du message dans la zone d'alerte
        $(".alert.ajax.alert-danger").find('.content').html(html);
        $(".alert.ajax.alert-danger").fadeIn(500);
    },

    cleanErrors: function() {

        // Suppression des messages d'erreurs en cours (champs & flash)
        $('.help-block').remove();
        $('.has-error').removeClass('has-error');
        $(".alert.ajax.alert-danger").find('.content').html('');
        $(".alert.ajax").hide();
    }
}

// Modulus.popuptexteditor = {

//     trigger: null,
//     callback: null,
//     instance: null,

//     init: function(options) {

//         if (!options.trigger)
//             console.log('Text editor - Le paramètre trigger est requis');

//         this.trigger     = options.trigger;
//         this.callback    = options.callback  ? options.callback : null;
//         this.setListeners();

//     },
//     setListeners: function() {

//         var self = this;

//         $(document).on('click', this.trigger, function() {

//             var el_trigger = this;

//             // On récupère un potentiel paramètre folder
//             $.fancybox({
//                 content: '<div id="popuptexteditor"></div>',
//                 helpers : {
//                     title: {
//                       type: 'outside',
//                       position: 'top'
//                     }
//                 },
//                 beforeLoad: function() {
//                     this.title = '<h3 class="text-center">Editeur de texte</h3>';
//                 },
//                 afterShow: function() {
//                     self.setTinyMCE(el_trigger);
//                 },
//                 afterClose: function() {
//                     self.instance.destroy();
//                 },
//                 closeBtn: true,
//                 width: 900,
//                 maxHeight: 700
//             });
//         });
//     },

//     setTinyMCE: function(trigger) {

//         var self = this;

//         //On applique le plugin
//         tinymce.init({
//             selector: selector,
//             height: 450,
//             branding: false,
//             language: 'fr_FR',
//             file_browser_callback : this.setBrowser,
//             plugins: [
//                 'advlist autolink lists link image charmap print preview anchor',
//                 'searchreplace visualblocks code fullscreen',
//                 'insertdatetime table contextmenu paste code textcolor colorpicker'
//             ],
//             toolbar: 'bold italic underline strikethrough | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image | forecolor backcolor | fullscreen',
//             setup: function (editor) {
//                 editor.on('change', function () {
//                     editor.save();
//                     // Lancement du callback
//                     if (self.callback) self.callback(trigger, $('#popuptexteditor').html());
//                 });
//             }
//         });
//     },

//     setBrowser: function(field_name, url, type, win) {
//         tinymce.activeEditor.windowManager.open({
//             file: '/backend/manager/elfinder', // Chemin vers iframe elfinder
//             title: 'Gestionnaire de fichiers',
//             width: 750,
//             height: 430,
//             resizable: 'no'
//         }, {
//             setUrl: function (url) {
//                 win.document.getElementById(field_name).value = url;
//             }
//         });
//         return false;
//     }
// };


Modulus.clipboard = {

    clipboard: null,

    /**
     * Methode d'initialisation du plugin Clipboard
     * @param  {String} target Class ou Id de l'élement à cibler
     * @return {null}
     */
    init: function(target) {

        // Suppression de l'event Bootstrap pour compatibilité avec clipboard
        $.fn.modal.Constructor.prototype.enforceFocus = function() {}; // Bootstrap 3;
        $.fn.modal.Constructor.prototype._enforceFocus = function() {} // Bootstrap 4;

        // Résoudre le conflit dans Jquery UI Tooltip avec Bootstrap Tootip
        //$.widget.bridge('uibutton', $.ui.button);



        this.clipboard = new Clipboard(target);
        this.setListeners();
    },
    setListeners: function() {

        var self = this;

        this.clipboard.on('success', function(evt) {

            self.success(evt.trigger);
        });

        this.clipboard.on('error', function(evt) {
            console.log(evt);
            self.error(evt.trigger);
        });
    },
    success: function(trigger) {
        $(trigger).tooltip({
            title: 'Copié !',
            trigger: 'manual',
        })
        .tooltip('show')
        .one('mouseleave', function() {
            $(this).tooltip('dispose');
        });
    },
    error: function(trigger) {
        $(trigger).tooltip({
            title: 'ERREUR INCONNUE !',
            trigger: 'manual',
        })
        .tooltip('show')
        .one('mouseleave', function() {
            $(this).tooltip('dispose');
        });
    }
}

Modulus.popup = {

    init: function() {
        this.setListeners();

        $("#popup-alert .alert").delay(10000).fadeOut(5000, function() {
            $(this).alert('close');
        });
    },

    setListeners: function() {
        var self = this;

        //Ecoute sur les boutons supprimer
        $(document).on('click', '.delete', function(event){

            event.preventDefault();
            var href_val = $(this).attr('data-href');
            self.delete(function() {

                document.location.href = href_val;
            })
        });

        $(document).on('click', '.confirm', function(event){

            event.preventDefault();
            var href_val = $(this).attr('data-href');
            self.confirm(function() {

                document.location.href = href_val;
            })
        });
    },

    nofound: function() {

        Swal.fire({
            title: "Attention !",
            text: "La page demandé n'a plus être trouvée...",
            icon: "warning",
            showCancelButton: true,
            confirmButtonColor: "#DD6B55",
            confirmButtonText: "Retour",
            cancelButtonColor: '#928a8a' ,
            cancelButtonText: "Annuler",
        }).then(function(result){
            if(result.value) {
                document.location.reload();
            }
        });
    },

    ajax: function() {
        Swal.fire({
            title: "Attention !",
            text: "Erreur de communication avec le serveur...",
            icon: "warning",
            showCancelButton: true,
            confirmButtonColor: "#DD6B55",
            confirmButtonText: "Retour",
            cancelButtonColor: '#928a8a' ,
            cancelButtonText: "Annuler",
        }).then(function(result){
            if(result.value) {
                document.location.reload();
            }
        });
    },
    confirm: function(callback) {
        Swal.fire({
            title: "Attention !",
            text: "Veuillez confirmer cette action",
            icon: "warning",
            showCancelButton: true,
            confirmButtonColor: "#DD6B55",
            confirmButtonText: "Confirmer",
            cancelButtonColor: '#928a8a' ,
            cancelButtonText: "Annuler",
        }).then(function(result){
            if(result.value) {
                callback();
            }
        });
    },
    delete: function(callback) {
        Swal.fire({
            title: "Attention !",
            text: "Voulez-vous vraiment supprimer cet élément ?",
            icon: "warning",
            showCancelButton: true,
            confirmButtonColor: "#DD6B55",
            confirmButtonText: "Supprimer",
            cancelButtonColor: '#928a8a' ,
            cancelButtonText: "Annuler",
        }).then(function(result){
            if(result.value) {
                callback();
            }
        });
    },
    alert: function(type, datas) {
        var html = '';
        html += '<div class="alert alert-' + type + ' alert-dismissible animate__animated animate__tada" role="alert">';
        html += '<button type="button" class="close" data-dismiss="alert" aria-label="Close">';
        html += '<span aria-hidden="true">×</span>';
        html += '</button>';

        switch (type) {
            case 'danger':
            case 'error' :
                html += '<i class="fa fa-times mr-2"></i>Erreur<br>';
                break;
            case 'success' :
                html += '<i class="fa fa-check mr-2"></i>';
                break;
            case 'warning' :
                html += '<i class="fa fa-exclamation-triangle mr-2"></i>Attention<br>';
                break;
            default :
                break;
        }

        if ($.isPlainObject(datas)) {
            $.each(datas, function(key, val) {
                if ($.isArray(val)) {
                    html += val[0] + '<br>';
                } else {
                    html += val;
                }
            });
        } else {
            html += datas;
        }
        html += '</div>';

        $('#popup-alert').html(html);

        $("#popup-alert .alert").delay(10000).fadeOut(5000, function() {
            $(this).alert('close');
        });

    }
};

// Modulus.position = {
//     init: function() {
//         var items_container = $('.table-sortable tbody').toArray();

//         var itemdrag = dragula(items_container, {
//             revertOnSpill: true,
//             direction: 'vertical',
//             moves: function (el, container, handle) {
//                 return $(handle).hasClass('position-wrap') ||
//                 $(handle).closest('.position-wrap').length != 0;
//             }
//         })
//         .on('drop', function (el, target, source, sibling) {

//             var $items = $(source).children();

//             // Récupération de la position min
//             position_min = null;
//             $items.each(function(key, item) {
//                 var position = $(item).find('.position').attr('data-sort');

//                 if (position_min == null || position < position_min) {
//                     position_min = position;
//                 }
//             });

//             // Création d'un tableau de position des IDs
//             var items_position = {};
//             var current_position = position_min;
//             $items.each(function(key, item) {
//                 var current_id = $(item).find('.position').attr('data-id');
//                 items_position[current_position] = current_id;
//                 current_position++;
//             });

//             // Envoi du classement au controleur
//             $.ajax({
//                 url        : document.location.href,
//                 type       : 'post',
//                 headers    : { 'X-CSRF-Token' : window.Laravel.csrfToken },
//                 async      : true,
//                 dataType   : 'json',
//                 data       : {position: items_position},
//                 success    : function(result){
//                     Modulus.popup.alert('success', result);

//                     // Moduluslication des modifications dans le DOM
//                     $.each(items_position, function(position, id) {
//                         $('.position[data-id="' + id + '"]')
//                             .attr('data-sort', position)
//                             .text(position);
//                     });
//                 },
//                 error      : function(result){ Modulus.popup.alert('error', result); },
//             });
//         });

//     },
// }

;$(function(){

    Modulus.initAll();
});

// Pour la compilation npm
module.exports = Modulus;
