/*

    Nécessite les inclusions suivantes : 
    <script src="{!! asset('packages/tinymce/tinymce.min.js') !!}" type="text/javascript"></script>
    <script src="{!! asset('packages/tinymce/jquery.tinymce.min.js') !!}" type="text/javascript"></script>

    Exemple d'utilisation : 

        texteditor_opts = {
            toolbar: 'bold italic underline strikethrough | bullist numlist outdent indent | link | forecolor backcolor | code',
            menubar:false,
            statusbar: false,
        }

        $('text-editor-simple').texteditor(texteditor_opts);
 */

;(function($)
{
    $.fn.texteditor = function(opts) {

        /**
         * Attributs de l'objet
         * @type {Any}
         */
        var attr = $.extend({
            menubar:true,
            statusbar: true,
            toolbar: 'bold italic underline strikethrough | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image | forecolor backcolor | fullscreen'
        }, opts);

        /**
         * Méthodes de l'object
         * @type {Object}
         */
        var Self = {};

        Self.init = function(el) {
            /**
             * Désactivation de la suppression de tinymce à l'initialisation : 
             * Cause : Impossible d'avoir plusieurs instance
             * Date : 20.01.2020
             */
            //tinymce.remove();

            $(el).tinymce({
                //selector: selector,
                branding: false,
                language: 'fr_FR',
                file_picker_callback : Self.setFilePicker,
                plugins: [
                    'advlist autolink lists link image charmap print preview anchor',
                    'searchreplace visualblocks code fullscreen',
                    'insertdatetime table contextmenu paste code textcolor colorpicker'
                ],
                menubar: attr.menubar,
                toolbar: attr.toolbar,
                statusbar: attr.statusbar,
                setup: function (editor) {
                    editor.on('change', function () {
                        editor.save();
                    });
                }
            });
        };

        Self.setFilePicker = function(callback, value, meta) {
            var myIfm, pStyle;
            var filemanager = '/backend/manager/elfinder';

            tinymce.activeEditor.windowManager.open({        
                title: 'Gestionnaire de fichiers',
                size: 'large',
                body: {
                    type: 'panel',
                    items: [{
                        type: 'htmlpanel',
                        html: '<iframe class="elfinder-manager" style="width:100%; height:100%;" src="' + filemanager + '"/>'
                    }]
                },
                buttons: []
            });

            tinymce.activeEditor.windowManager._elfinderCallBack = function (file, fm) {
                var url, reg, info;

                // URL normalization
                url = fm.convAbsUrl(file.url);
                
                // Make file info
                info = file.name + ' (' + fm.formatSize(file.size) + ')';

                // Provide file and text for the link dialog
                if (meta.filetype == 'file') {
                    callback(url, {text: info, title: info});
                }

                // Provide image and alt text for the image dialog
                if (meta.filetype == 'image') {
                    callback(url, {alt: info});
                }

                // Provide alternative source and posted for the media dialog
                if (meta.filetype == 'media') {
                    callback(url);
                }
            };
        }


        /**
         * Return   Applique le plugin sur tous les élements selectionnés
         */
        return this.each(function() {
            Self.init(this);

        });
    }
})(jQuery);