/**
      ____                                          ____  _   _  _
     / ___|___  _ __ ___  _ __   ___  ___  ___ _ __| __ )| |_| || |
    | |   / _ \| '_ ` _ \| '_ \ / _ \/ __|/ _ \ '__|  _ \| __| || |_
    | |__| (_) | | | | | | |_) | (_) \__ \  __/ |  | |_) | |_|__   _|
     \____\___/|_| |_| |_| .__/ \___/|___/\___|_|  |____/ \__|  |_|
                         |_|
 *
 *
 * @author: Sylvain CARRE
 * @email : contact@goldenscarab.fr
 * date   : 07/08/2018
 *
 *
 * Utilisation :
 *
 * HTML :
 * <textarea class="composer-bt4" name="content" rows="3"></textarea>
 *
 * JS :
 * $('.composer-bt4').composerBt4();
 */

;(function($)
{
    $.fn.composerBt4 = function(opts) {

        /**
         * Attributs de l'objet
         * @type {Any}
         */
        var attr = $.extend({
            composerId  : 'composer-bt4',
            toolbarClass: 'composer-toolbar',
            overlayClass: 'composer-overlay',
            defaultValue: ['container', 'row', 'col']
        }, opts);

        /**
         * Méthodes de l'object
         * @type {Object}
         */
        var Self = {};

        Self.init = function(el) {

            var value, $composer;

            $(el).hide();
            value = $(el).val();

            // Création de l'espace de travail
            $composer = $('<div>').attr('id', attr.composerId);
            $composer.append(Self.make.workspace);

            // Ajout de la valeur du champs
            $composer.find('.workspace').append(value);

            $(el).after($composer);

            Self.toolbar.addMissing();

            Self.addRequiredClassesOnComponents();

            Self.listeners(el);

            if (!value) {
                Self.addDefaultValue($composer);
            }

            Self.position(el);
        };

        Self.addDefaultValue = function(el) {

            $.each(attr.defaultValue, function(key, component) {
                $btn = $(el).find('.tool-add[data-declinations="'+component+'"]');
                if ($btn.length == 0) {
                    $btn = $(el).find('.tool-add[data-component="'+component+'"]');
                }

                $($btn).trigger('click');
            });
        };

        Self.position = function(el) {

            // Liste des containers ayant des éléments déplacable avec le grep
            var composer_containers = ['.workspace', '.container, .container-fluid', '.row']

            $.each(composer_containers, function(key, item) {
                dragula($(item).toArray(), {
                    revertOnSpill: true,
                    removeOnSpill :  false,
                    moves: function (el, container, handle) {

                        var classes_required = $(container).attr('class');
                        var current_container = $(handle).closest('.' + attr.toolbarClass).parent().parent();

                        classes_required = '.' + classes_required.replace(/\s/g, ', .');

                        return $(current_container).is(classes_required) && $(handle).hasClass('tool-move');
                    },
                    accepts: function (el, target, source, sibling) {
                        var is_after_toolbar = $(el).prevAll().hasClass(attr.toolbarClass);
                        var is_in_composer = $(el).closest('#' + attr.composerId).length > 0;

                        return is_after_toolbar && is_in_composer;
                    },
                });
            });
        };

        Self.addRequiredClassesOnComponents = function() {

            $.each(requiredClassesOnComponents, function(i, component) {
                $('#' + attr.composerId).find(component.selector).addClass(component.classes);
            });
        }

        Self.getContentCleaned = function($content) {

            $content.find('.' + attr.toolbarClass).remove();
            $content.find('.' + attr.overlayClass).remove();
            $content.find('.overflight').removeClass('overflight');
            $content.find('.overlable').removeClass('overlable');

            return $content;
        };

        Self.listeners = function(el) {

            Self.listeners.toolbar();
            Self.listeners.overlay();

            // Au changement mise à jour du champs
            var observer = new MutationObserver(function(e) {
                var $content = $('#composer-bt4 .workspace').clone();
                $content = Self.getContentCleaned($content);
                $(el).val($content.html());
            });
            observer.observe($('#' + attr.composerId)[0], {
                characterData: true,
                childList: true,
                subtree: true
            });

        };

        Self.listeners.toolbar = function() {
            // Ajout d'un élément
            $('#' + attr.composerId).on('click', '.tool-add', function() {
                var $new_element;
                var component_name   = $(this).attr('data-component');
                var declinations_name = $(this).attr('data-declinations');

                // Récupération du template à ajouter
                if (declinations_name) {
                    $new_element = $(componentsBt4[component_name].declinations[declinations_name].template);
                } else {
                    $new_element = $(componentsBt4[component_name].template);
                }

                // Positionnement sur le conteneur concerné
                $container = $(this).closest('.' + attr.toolbarClass).parent();

                $container.append($new_element);

                // Réglage de la position du scroll sur le nouvel élément
                var offset     = $new_element.offset().top;
                var height     = $new_element.height();
                var win_height = $(window).height();
                var new_scroll;

                if (height < win_height) {
                    new_scroll = offset - ((win_height / 2) - (height / 2));
                }
                else {
                    new_scroll = offset;
                }
                scroll(0, new_scroll)

                // Mise à jour des toolbars et classes
                Self.toolbar.addMissing();
                Self.addRequiredClassesOnComponents();
            });


            // Action sur l'élément
            $('#' + attr.composerId).on('click', '.tool-action', function() {
                var $component = $(this).closest('.' + attr.toolbarClass).parent();

                var action = $(this).attr('data-action');
                var popup_name;

                if (action == 'delete') {
                    if (confirm("Veuillez confirmer la suppression de cet élément")) {
                        $component.remove();
                    }
                } else if (action.match(/^edit-.*/)) {
                    popup_name = action.replace(/^edit-/, '');

                    switch(popup_name) {
                        case 'source' :
                            Self.popup.codesource($component, function(html) {
                                $component.html(html);
                                Self.toolbar.addMissing();
                                Self.addRequiredClassesOnComponents();
                                Self.position($component.closest('#' + attr.composerId));
                            });
                            break;

                        default :
                            Self.popup[popup_name]($component, function() {
                                Self.toolbar.refresh();
                            });
                    }

                } else if (action.match(/^change-.*/)) {

                }
            });
        };

        Self.listeners.overlay = function() {

            // Au survole des composants superposable
            $('#' + attr.composerId).on('mouseover', '.overlable', function() {

                $('.overlable').trigger('mouseout');
                $(this).addClass('overflight');

                // Récupération de la classe d'identification du composant
                var component_name = $(this).attr('class').split(' ')[0];
                var $overlay = Self.make.overlay.create(component_name);

                // Positionnement de l'overlay
                var component_width  = $(this).outerWidth(true);
                var component_height = $(this).outerHeight(true);
                var component_top    = $(this).get(0).offsetTop;
                var component_left   = $(this).get(0).offsetLeft;
                var overlay_top      = component_top + (component_height / 2);
                var overlay_left     = component_left + (component_width / 2);
                $overlay.css({top: overlay_top + 'px', left: overlay_left + 'px'});

                // Ajout de l'overlay
                $(this).after($overlay);
            });

            // Suppression des overlays
            $(document).on('mouseout', '.overlable', function(ev) {
                var $overlay = $(ev.relatedTarget).closest('div.' + attr.overlayClass);

                // Si l'élément survolé n'est pas un enfant de l'overlay on le supprime
                if ($overlay.length == 0) {
                    $('div.' + attr.overlayClass).remove();
                    $('.overflight').removeClass('overflight');
                }
            });

            // Action sur l'élément
            $('#' + attr.composerId).on('click', '.overlay-action', function() {
                var $component = $(this).closest('.' + attr.overlayClass).prev();

                var action = $(this).attr('data-action');
                var popup_name;

                if (action == 'delete') {
                    if (confirm("Veuillez confirmer la suppression de cet élément")) {
                        $component.remove();
                    }
                } else if (action.match(/^edit-.*/)) {
                    popup_name = action.replace(/^edit-/, '');

                    if (popup_name == 'source') {
                        Self.popup.codesource($component, function(html) {
                            $component.html(html);
                            Self.toolbar.addMissing();
                            Self.addRequiredClassesOnComponents();
                            Self.position($component.closest('#' + attr.composerId));
                        });
                    } else {
                        Self.popup[popup_name]($component, function() {
                            Self.toolbar.refresh();
                        });
                    }
                }
            });
        };

        Self.popup = {};

        Self.popup.id = function($el, callback) {

            var $form    = $('<div>').addClass('col').append($(layoutBt4['field-id'])).append($(layoutBt4['fancy-save']));
            var $content = $('<div>').addClass('form-row').append($form);

            // Fancy box
            $.fancybox({
                content: $content,
                width: '600',
                height: '120',
                scrolling : 'no',
                autoSize: false,
                helpers : {
                    overlay : {closeClick: false}
                },
                afterShow: function() {
                    // Remplir le champs
                    $('#field-id').val($el.attr('id'));
                },
                beforeClose: function() {
                    // Enregistrer le champs
                    $el.attr('id', $('#field-id').val());
                    callback();
                }
            });
        };

        Self.popup.class = function($el, callback) {

            var $form    = $('<div>').addClass('col').append($(layoutBt4['field-class'])).append($(layoutBt4['fancy-save']));
            var $content = $('<div>').addClass('form-row').append($form);

            // Fancy box
            $.fancybox({
                content: $content,
                width: '600',
                height: '120',
                scrolling : 'no',
                autoSize: false,
                helpers : {
                    overlay : {closeClick: false}
                },
                afterShow: function() {
                    // Remplir le champs
                    $('#field-class').val($el.attr('class'));
                },
                beforeClose: function() {
                    // Enregistrer le champs
                    $el.attr('class', $('#field-class').val());
                    callback();
                }
            });
        };

        Self.popup.style = function($el, callback) {

            var $form    = $('<div>').addClass('col')
                                .append($('<h4>').text('Couleurs'))
                                .append($(layoutBt4['field-color']))
                                .append($('<h4>').text('Marges'))
                                .append($(layoutBt4['fields-margin']))
                                .append($('<h4>').text('Décalages'))
                                .append($(layoutBt4['fields-padding']))
                                .append($(layoutBt4['fancy-save']));
            var $content = $('<div>').addClass('form-row').append($form);

            // Fancy box
            $.fancybox({
                content: $content,
                scrolling : 'no',
                autoSize: true,
                helpers : {
                    title: {
                      type: 'outside',
                      position: 'top'
                    },
                    overlay : {closeClick: false}
                },
                beforeLoad: function() {
                    this.title = '<h3 class="text-center">Style de l\'élément</h3>';
                },
                afterShow: function() {
                    // Remplir les champs
                    $('#field-color-background').val(hexc($el.css('background-color')));
                    $('#field-color-text').val(hexc($el.css('color')));

                    $('#field-margin-top').val($el.css('margin-top'));
                    $('#field-margin-right').val($el.css('margin-right'));
                    $('#field-margin-bottom').val($el.css('margin-bottom'));
                    $('#field-margin-left').val($el.css('margin-left'));

                    $('#field-padding-top').val($el.css('padding-top'));
                    $('#field-padding-right').val($el.css('padding-right'));
                    $('#field-padding-bottom').val($el.css('padding-bottom'));
                    $('#field-padding-left').val($el.css('padding-left'));
                },
                beforeClose: function() {
                    // Enregistrer les champs si changement détécté
                    /* Couleurs */
                    if ($('#field-color-background').val() != hexc($el.css('background-color')))
                        $el.css('background-color', $('#field-color-background').val());

                    if ($('#field-color-text').val() != hexc($el.css('color')))
                        $el.css('color', $('#field-color-text').val());

                    /* Marges */
                    if ($('#field-margin-top').val() != $el.css('margin-top'))
                        $el.css('margin-top', $('#field-margin-top').val());

                    if ($('#field-margin-right').val() != $el.css('margin-right'))
                        $el.css('margin-right', $('#field-margin-right').val());

                    if ($('#field-margin-bottom').val() != $el.css('margin-bottom'))
                        $el.css('margin-bottom', $('#field-margin-bottom').val());

                    if ($('#field-margin-left').val() != $el.css('margin-left'))
                        $el.css('margin-left', $('#field-margin-left').val());

                    /* Décalages */
                    if ($('#field-padding-top').val() != $el.css('padding-top'))
                        $el.css('padding-top', $('#field-padding-top').val());

                    if ($('#field-padding-right').val() != $el.css('padding-right'))
                        $el.css('padding-right', $('#field-padding-right').val());

                    if ($('#field-padding-bottom').val() != $el.css('padding-bottom'))
                        $el.css('padding-bottom', $('#field-padding-bottom').val());

                    if ($('#field-padding-left').val() != $el.css('padding-left'))
                        $el.css('padding-left', $('#field-padding-left').val());

                    callback();
                }
            });

            function hexc(colorval) {
                var color_ = [], parts, color;

                parts = colorval.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);
                if (parts == null) {
                    parts = colorval.match(/^rgba\((\d+),\s*(\d+),\s*(\d+),\s*(\d+)\)$/);
                }
                for (var i = 1; i <= 3; ++i) {
                    color_[i] = parseInt(parts[i]).toString(16);
                    if (color_[i].length == 1) color_[i] = '0' + color_[i];
                }
                color = '#' + color_.join('');

                return color;
            }
        };

        Self.popup.element = function($el, callback) {

            var $label   = $('<label>').text('Modification de l\'élément');
            var $editor  = $('<div>').attr('id', 'editor-code')
                                    .attr('data-target', '#field-code')
                                    .attr('data-language', 'html')
                                    .css({minHeight: '50px'});
            var $field   = $('<textarea>').attr('id', 'field-code').css({'display': 'none'});
            var $form    = $('<div>').addClass('col')
                                    .append($label)
                                    .append($editor)
                                    .append($field)
                                    .append($(layoutBt4['fancy-save']));
            var $content = $('<div>').addClass('form-row').append($form);
            var html, params, $element;

            // Fancy box
            $.fancybox({
                content: $content,
                width: '900',
                height: '120',
                scrolling : 'no',
                autoSize: false,
                helpers : {
                    overlay : {closeClick: false}
                },
                afterShow: function() {
                    // Remplir le champs avec le code HTML de l'élément
                    html = $el.clone().html('').get(0).outerHTML;
                    $('#editor-code').text(html);
                    $('#field-code').val(html);
                    $('#editor-code').codeeditor();
                },
                beforeClose: function() {
                    // Enregistrer le champs
                    html = $('#field-code').val();

                    // Extraction des paramètres du code
                    var params = html.match(/(\S+)=["']?((?:.(?!["']?\s+(?:\S+)=|[>"']))+.)["']?/gmi)

                    // Mise à jour des paramètres dans la balise source
                    $.each(params, function(key, param) {
                        let param_ = param.split('=');
                        let param_name = param_[0];
                        let param_value = param_[1].replace(/[\"\']/g, '');

                        $el.attr(param_name, param_value);
                    });

                    callback(html);
                }
            });
        };

        Self.popup.codesource = function($el, callback) {

            var $label   = $('<label>').text('Contenu HTML de l\'élément');
            var $editor  = $('<div>').attr('id', 'editor-code')
                                    .attr('data-beautiful', true)
                                    .attr('data-target', '#field-code')
                                    .attr('data-language', 'html')
                                    .css({minHeight: '618px'});
            var $field   = $('<textarea>').attr('id', 'field-code').css({'display': 'none'});
            var $form    = $('<div>').addClass('col')
                                    .append($label)
                                    .append($editor)
                                    .append($field)
                                    .append($(layoutBt4['fancy-save']));
            var $content = $('<div>').addClass('form-row').append($form);
            var html, params, $element;

            // Fancy box
            $.fancybox({
                content: $content,
                width: '900',
                height: '680',
                scrolling : 'no',
                autoSize: false,
                helpers : {
                    overlay : {closeClick: false}
                },
                afterShow: function() {
                    // Remplir le champs avec le code HTML de l'élément
                    $html = Self.getContentCleaned($el.clone());
                    $('#editor-code').text($html.html());
                    $('#field-code').val($html.html());
                    $('#editor-code').codeeditor();
                },
                beforeClose: function() {
                    // Enregistrer le champs
                    html = $('#field-code').val();
                    callback(html);
                }
            });
        };

        Self.popup.imagefinder = function($image) {
            elfinder = null;

            $.fancybox({
                content: '<div id="imagefinder"></div>',
                helpers : {
                    title: {
                      type: 'outside',
                      position: 'top'
                    },
                    overlay : {closeClick: false}
                },
                beforeLoad: function() {
                    this.title = '<h3 class="text-center">Gestionnaire d\'images</h3>';
                },
                afterShow: function() {
                    elfinder = $('#imagefinder').elfinder({
                        url: '/elfinder/connector',
                        lang: 'fr',
                        disabled: ['extract', 'archive', 'search'],
                        requestType: 'get',
                        resizable: false,
                        //width: '100%',
                        height: 530,
                        commands : ['open', 'reload', 'home', 'up', 'back', 'forward', 'getfile', 'quicklook', 'download', 'rm', 'rename', 'mkdir', 'upload', 'info', 'resize', 'sort'],
                        onlyMimes: ['image'],
                        uiOptions : {
                            toolbar : [
                                ['back', 'forward'],['reload'],['home', 'up'],['mkdir', 'mkfile', 'upload'],['open', 'download', 'getfile'],['info'],['quicklook'],['rm', 'rename', 'resize'],['search'],
                            ]
                        },
                        customData: {
                            _token: window.Laravel.csrfToken
                        },
                        commandsOptions : {
                            getfile: {
                                onlyURL: false,
                                multiple: false,
                                folders: false,
                            }
                        },
                        getFileCallback: function(url, instance) {

                            // Modification de l'élement
                            $image.attr('src', '/' + url.path);

                            $.fancybox.close();
                        }

                    }).elfinder('instance');
                },
                afterClose: function() {
                    elfinder.destroy();
                },
                autoSize: false,
                padding: [10, 5, 10, 10],
                scrolling: 'no',
                closeBtn: true,
                width: '100%',
                maxWidth: 800,
                height: 'auto'
            });
        };

        Self.popup.texteditor = function($article) {
            $.fancybox({
                content: '<div id="texteditor" style="height: 550px;">' + $article.html() + '</div>',
                helpers : {
                    title: {
                      type: 'outside',
                      position: 'top'
                    },
                    overlay : {closeClick: false}
                },
                beforeLoad: function() {
                    this.title = '<h3 class="text-center">Editeur de texte</h3>';
                },
                afterShow: function() {
                    tinymce.init({
                        selector: '#texteditor',
                        width: '99%',
                        height: 450,
                        branding: false,
                        language: 'fr_FR',
                        file_picker_callback: setFilePicker,
                        plugins: [
                            'advlist autolink lists link image charmap print preview anchor',
                            'searchreplace visualblocks code fullscreen',
                            'insertdatetime table contextmenu paste code textcolor colorpicker'
                        ],
                        toolbar: 'bold italic underline strikethrough | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image | forecolor backcolor | fullscreen',
                        setup: function (editor) {
                            editor.on('change', function () {
                                editor.save();
                                $article.html($('#texteditor').html());
                            });
                        }
                    });
                },
                afterClose: function() {
                   tinymce.get('texteditor').remove();
                },
                autoSize: false,
                padding: [10, 5, 10, 10],
                scrolling: 'no',
                closeBtn: true,
                width: '100%',
                maxWidth: 960,
                height: 'auto'
            });

            function setFilePicker(callback, value, meta) {
                var myIfm, pStyle;
                var filemanager = '/backend/manager/elfinder';

                tinymce.activeEditor.windowManager.open({
                    title: 'Gestionnaire de fichiers',
                    size: 'large',
                    body: {
                        type: 'panel',
                        items: [{
                            type: 'htmlpanel',
                            html: '<iframe class="elfinder-manager" style="width:100%; height:100%;" src="' + filemanager + '"/>'
                        }]
                    },
                    buttons: []
                });

                tinymce.activeEditor.windowManager._elfinderCallBack = function (file, fm) {
                    var url, reg, info;

                    // URL normalization
                    url = fm.convAbsUrl(file.url);

                    // Make file info
                    info = file.name + ' (' + fm.formatSize(file.size) + ')';

                    // Provide file and text for the link dialog
                    if (meta.filetype == 'file') {
                        callback(url, {text: info, title: info});
                    }

                    // Provide image and alt text for the image dialog
                    if (meta.filetype == 'image') {
                        callback(url, {alt: info});
                    }

                    // Provide alternative source and posted for the media dialog
                    if (meta.filetype == 'media') {
                        callback(url);
                    }
                };
            }
        };

        Self.toolbar = {};

        Self.toolbar.addMissing = function() {
            var components = [];

            // Récupération des composants disponibles
            $.each(componentsBt4, function(key, item) {
                components.push(key);

                $.each(item.declinations, function(key_, item_) {

                    if ($.inArray(key_, components) < 0) {
                        components.push(key_);
                    }
                });
            });

            // Ajout des barres d'outils
            $.each(components, function(key, component) {

                // Si barre d'outil non existante
                $('#' + attr.composerId).find('.' + component).each(function(key, item) {
                    if ($(item).find('.' + attr.toolbarClass).length == 0) {
                        var $toolbar = Self.make.toolbar.create($(item));
                        $(item).prepend($toolbar);
                    }
                });

            });
        };

        Self.toolbar.refresh = function() {

            var $component;

            // Recherche des barres d'outils
            $('.' + attr.toolbarClass).each(function (key, item) {
                $component = $(item).parent();

                // On remplace la barre d'outil
                $(item).replaceWith(Self.make.toolbar.create($component));
            });
        };

        Self.make = {};

        Self.make.workspace = function() {
            var $workspace = $('<div>').addClass('workspace');
            return $workspace;
        };

        Self.make.toolbar = {};

        Self.make.toolbar.create = function($target_component) {

            var $toolbar, $toolbar_, $tool, $icon, $button, $info_id, $info_classes, $move;
            var toolbar_required = false, default_required = false, tool_required = true;
            var target_component_name;

            target_component_name = $target_component.attr('class').split(' ')[0];

            // Création du layout
            $toolbar_ = $('<div>').addClass(attr.toolbarClass);
            $toolbar_.attr('data-component', target_component_name);

            $toolbar_add = $('<div>').addClass('toolbar-add');
            $toolbar_change = $('<div>').addClass('toolbar-change');

            // Parcours des composants
            $.each(componentsBt4, function(component_name, component_data) {
                // Test des dépendances pour ajout de la barre d'outil
                toolbar_required = false;
                $.each(component_data.belongs, function(key, belong) {

                    if((new RegExp('^' + belong)).test(target_component_name)) {
                        toolbar_required = true;
                        default_required = true;
                    }

                });


                // Ajout des composants de dépendances
                if (toolbar_required) {

                    $icon   = $(layoutBt4['fa-icon']).addClass(component_data.icon);

                    if (component_data.declinations.length == 0) {
                        $button = $(layoutBt4['btn-xs'])
                                        .addClass('tool-add')
                                        .attr('data-component', component_name)
                                        .attr('data-action', component_data.action)
                                        .attr('title', component_data.label)
                                        .append($icon);
                        $tool = $button;
                    } else {
                        var $dropdown = $(layoutBt4['dropdown-xs']).attr('title', component_data.label);
                        $dropdown.addClass('dropdown-add')
                        $dropdown.find('button').append($icon);

                        $.each(component_data.declinations, function(declinations_name, declinations_data) {
                            if (declinations_data.label) {
                                var item_dropdown = $('<button class="dropdown-item tool-add" type="button">')
                                                        .attr('data-component', component_name)
                                                        .attr('data-declinations', declinations_name)
                                                        .html(declinations_data.label);

                            }
                            $dropdown.find('.dropdown-menu').append(item_dropdown);
                        });
                        $tool = $dropdown;
                    }

                    $toolbar_add.append($tool);
                }

            });

            // Ajout des outils par defaut
            if (default_required) {

                if (target_component_name != 'workspace') {

                    // Ajout de l'icone de déplacement
                    $move = $(layoutBt4['fa-icon']).addClass('fa-arrows tool-move');
                    $toolbar_add.append($move);

                    // Ajout des infos du bloc
                    if ($target_component.attr('id')) {
                        $info_id = $(layoutBt4['badge']).addClass('badge-warning tool-info').text('#' + $target_component.attr('id'));
                    }
                    $info_classes = $(layoutBt4['badge']).addClass('badge-info tool-info').text('.' + $target_component.attr('class').replace(/ /g, '.'));
                    $toolbar_add.append($info_id);
                    $toolbar_add.append($info_classes);
                }

                // Parcour de la liste des outils dispo
                $.each(defaultToolsBt4, function (key, tool) {

                    // Test si l'outils doit être ajouté à cet élément
                    tool_required = true;
                    $.each(tool.excepts, function(i, except) {
                        if((new RegExp('^' + except)).test(target_component_name)) {
                            tool_required = false;
                        }
                    });

                    // Ajout de l'outil
                    if (tool_required) {
                        $button = $(layoutBt4['btn-xs'])
                                        .addClass('tool-action')
                                        .attr('data-action', tool.action)
                                        .attr('title', tool.title)
                                        .append(tool.label);
                        $toolbar_change.append($button);
                    }
                });
            }

            // Affichage de la toolbar
            if (toolbar_required || default_required) {
                $toolbar_.append($toolbar_add);
                $toolbar_.append($toolbar_change);
                $toolbar = $toolbar_;
            }

            return $toolbar;
        };

        Self.make.overlay = {};

        Self.make.overlay.create = function(target_component_name) {
            var $overlay, $overlay_;
            $overlay = $overlay_ = $('<div>').addClass(attr.overlayClass);

            $.each(overlayToolsBt4, function (key, tool) {
                // Test si l'outils doit être ajouté à cet élément
                tool_required = false;
                $.each(tool.onlys, function(i, only) {
                    if((new RegExp('^' + only)).test(target_component_name)) {
                        tool_required = true;
                    }
                });

                // Ajout de l'outil
                if (tool_required) {
                    $button = $(layoutBt4['btn-circle'])
                                    .removeClass('btn-default')
                                    .addClass('overlay-action overlay-btn btn-success')
                                    .attr('data-action', tool.action)
                                    .attr('title', tool.title)
                                    .append(tool.label);
                    $overlay_.append($button);
                }
            });

            // Ajout dees outils par defaut
            $.each(defaultToolsBt4, function (key, tool) {
                // Test si l'outils doit être ajouté à cet élément
                tool_required = true;
                $.each(tool.excepts, function(i, except) {
                    if((new RegExp('^' + except)).test(target_component_name)) {
                        tool_required = false;
                    }
                });

                // Ajout de l'outil
                if (tool_required) {
                    $button = $(layoutBt4['btn-circle'])
                                    .addClass('overlay-action')
                                    .attr('data-action', tool.action)
                                    .attr('title', tool.title)
                                    .append(tool.label);
                    $overlay_.append($button);
                }
            });


            // Invertion de l'ordre des élements
            //$overlay_.children().each(function(key, item){$overlay.prepend(item)})

            return $overlay;
        };

        Self.overlay = {}

        Self.overlay.addMissing = function() {

            var components = [], $overlay;

            // Récupération des composants disponibles
            $.each(componentsBt4, function(key, item) {
                if (item.overlay) {
                    components.push(key);
                }
            });

            // Ajout des barres d'outils
            $.each(components, function(key, component) {

                // Si barre d'outil non existante
                $('#' + attr.composerId).find('.' + component).each(function(key, item) {
                    if ($(item).find('.' + attr.overlayClass).length == 0) {
                        $overlay = Self.make.overlay.create($(item));
                        $(item).prepend($overlay);
                    }
                });

            });
        }


        /**
         * Return   Retourne le plugin sur tous les élements selectionnés
         */
        return this.each(function() {
            Self.init(this);

        });
    }

    /**
     * Classes requises sur les composants
     * @type {Array}
     */
    var requiredClassesOnComponents = [
        {
            selector: 'img',
            classes : 'image overlable'
        },
        {
            selector: 'article',
            classes : 'article overlable'
        }
    ]

    /**
     * Modèle Bootstrap de base
     * @type {Object}
     */
    var layoutBt4 = {
        'btn-xs' : '<button class="btn btn-default btn-xs" type="button">',
        'btn-circle' : '<button class="btn btn-default btn-xs btn-circle" type="button">',
        'fa-icon' : '<i class="fa">',
        'dropdown-xs' : `<div class="dropdown"><button class="btn btn-default btn-xs" type="button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false"></button><div class="dropdown-menu"></div></div>`,
        'badge' : '<span class="badge"></badge>',
        'field-id': '<div class="form-group"><label>ID de l\'élément</label><div class="input-group"><div class="input-group-prepend"><span class="input-group-text" id="basic-addon1">#</span></div><input type="text" id="field-id" class="form-control"></div></div>',
        'field-class': '<div class="form-group"><label>Classes de l\'élément</label><input type="text" id="field-class" class="form-control"></div>',
        'field-color': '<div class="form-row"><div class="col-sm-6"><div class="form-group"><label>Arrière plan</label><input type="color" id="field-color-background" class="form-control" /></div></div><div class="col-sm-6"><div class="form-group"><label>Texte</label><input type="color" id="field-color-text" class="form-control"/></div></div></div>',
        'fields-margin': '<div class="row justify-content-center"><div class="col-2 mb-2"><input class="form-control form-control-sm" id="field-margin-top" placeholder="10px" type="text"/></div></div><div class="row justify-content-center"><div class="col-2"><input class="form-control form-control-sm" id="field-margin-left" placeholder="10px" type="text"/></div><div class="col-2" style="border: 3px solid #ff0000; height: 30px; text-align: center; color:#ff0000;">Élément</div><div class="col-2"><input class="form-control form-control-sm" id="field-margin-right" placeholder="10px" type="text"/></div></div><div class="row justify-content-center"><div class="col-2 mt-2"><input class="form-control form-control-sm" id="field-margin-bottom" placeholder="10px" type="text"/></div></div>',
        'fields-padding': '<div class="element" style="border: 3px solid #ff0000; text-align: center; color:#ff0000; padding: 5px; margin-bottom: 15px;"><div class="row justify-content-center"><div class="col-2"><input class="form-control form-control-sm" id="field-padding-top" placeholder="10px" type="text"/></div></div><div class="row justify-content-between"><div class="col-2"><input class="form-control form-control-sm" id="field-padding-left" placeholder="10px" type="text"/></div><div class="col-2">Élément</div><div class="col-2"><input class="form-control form-control-sm" id="field-padding-right" placeholder="10px" type="text"/></div></div><div class="row justify-content-center"><div class="col-2"><input class="form-control form-control-sm" id="field-padding-bottom" placeholder="10px" type="text"/></div></div></div>',
        'field-code': '<div class="form-group"><label>Code HTML de l\'élement</label><textarea id="field-code" class="form-control" rows="3" data-language="html"></textarea>',
        'fancy-save': '<button class="btn btn-xs btn-success" type="button" onclick="$.fancybox.close();">Enregistrer</button>'
    };

    /**
     * Outils par défaut
     * @type {Array}
     */
    var defaultToolsBt4 = [
        {
            label: '<i class="fa fa-hashtag"></i>',
            action: 'edit-id',
            title: "Édition de l'ID de l'élément",
            excepts: ['workspace']
        },
        {
            label: '<i class="fa fa-tag"></i>',
            action: 'edit-class',
            title: "Édition des classes de l'élément",
            excepts: ['workspace']
        },
        {
            label: '<i class="fa fa-paint-brush"></i>',
            action: 'edit-style',
            title: "Édition du style de l'élément",
            excepts: ['workspace']
        },
        {
            label: '<i class="fa fa-code"></i>',
            action: 'edit-source',
            title: "Édition du contenu HTML de l'élément",
            excepts: ['image']
        },
        {
            label: '<i class="fa fa-trash"></i>',
            action: 'delete',
            title: "Suppression de l'élement",
            excepts: ['workspace']
        },
    ];

    /**
     * Outil sur overlay
     * @type {Array}
     */
    var overlayToolsBt4 = [
        {
            label: '<i class="fa fa-pencil-square-o"></i>',
            action: 'edit-texteditor',
            title: "Modification de l'élement",
            onlys: ['article']
        },
        {
            label: '<i class="fa fa-pencil-square-o"></i>',
            action: 'edit-imagefinder',
            title: "Modification de l'élement",
            onlys: ['image']
        },
    ];

    /**
     * Composants de construction
     * @type {Object}
     */
    var componentsBt4 = {
        'workspace': {},
        'container': {
            label: 'Ajouter un conteneur',
            icon: 'fa-window-maximize',
            belongs: ['workspace'],
            declinations: {
                'container': {
                    label: '<i class="fa fa-list-alt mr-2"></i>Conteneur boxé',
                    template: '<div class="container">',
                },
                'container-fluid': {
                    label: '<i class="fa fa-arrows-h mr-2"></i>Conteneur étendu',
                    template: '<div class="container-fluid">',
                }
            },
            help: "https://getbootstrap.com/docs/4.3/layout/overview/#containers"
        },
        'row': {
            label: 'Ajouter une ligne',
            icon: 'fa-minus',
            template: '<div class="row">',
            belongs: ['container', 'container-fluid', 'col.*', 'w-100'],
            declinations: [],
            help: "https://getbootstrap.com/docs/4.3/content/tables/#striped-rows"
        },
        'col': {
            label: 'Ajouter un bloc colonne',
            icon: 'fa-columns',
            belongs: ['row'],
            declinations: {
                'col-1': {
                    label: '<i class="fa fa-list mr-2"></i>1 colonne',
                    template: '<div class="col-1">',
                },
                'col-sm-1': {},
                'col-md-1': {},
                'col-lg-1': {},
                'col-xl-1': {},

                'col-2': {
                    label: '<i class="fa fa-list mr-2"></i>2 colonnes',
                    template: '<div class="col-2">',
                },
                'col-sm-2': {},
                'col-md-2': {},
                'col-lg-2': {},
                'col-xl-2': {},

                'col-3' : {
                    label: '<i class="fa fa-list mr-2"></i>3 colonnes',
                    template: '<div class="col-3">',
                },
                'col-sm-3': {},
                'col-md-3': {},
                'col-lg-3': {},
                'col-xl-3': {},

                'col-4': {
                    label: '<i class="fa fa-th-list mr-2"></i>4 colonnes',
                    template: '<div class="col-4">',
                },
                'col-sm-4': {},
                'col-md-4': {},
                'col-lg-4': {},
                'col-xl-4': {},

                'col-5': {
                    label: '<i class="fa fa-th-list mr-2"></i>5 colonnes',
                    template: '<div class="col-5">',
                },
                'col-sm-5': {},
                'col-md-5': {},
                'col-lg-5': {},
                'col-xl-5': {},

                'col-6': {
                    label: '<i class="fa fa-th-large mr-2"></i>6 colonnes',
                    template: '<div class="col-6">',
                },
                'col-sm-6': {},
                'col-md-6': {},
                'col-lg-6': {},
                'col-xl-6': {},

                'col-7': {
                    label: '<i class="fa fa-th-list fa-flip-horizontal mr-2"></i>7 colonnes',
                    template: '<div class="col-7">',
                },
                'col-sm-7': {},
                'col-md-7': {},
                'col-lg-7': {},
                'col-xl-7': {},

                'col-8': {
                    label: '<i class="fa fa-th-list fa-flip-horizontal mr-2"></i>8 colonnes',
                    template: '<div class="col-8">',
                },
                'col-sm-8': {},
                'col-md-8': {},
                'col-lg-8': {},
                'col-xl-8': {},

                'col-9': {
                    label: '<i class="fa fa-th-list fa-flip-horizontal mr-2"></i>9 colonnes',
                    template: '<div class="col-9">',
                },
                'col-sm-9': {},
                'col-md-9': {},
                'col-lg-9': {},
                'col-xl-9': {},

                'col-10': {
                    label: '<i class="fa fa-list fa-flip-horizontal mr-2"></i>10 colonnes',
                    template: '<div class="col-10">',
                },
                'col-sm-10': {},
                'col-md-10': {},
                'col-lg-10': {},
                'col-xl-10': {},

                'col-11': {
                    label: '<i class="fa fa-list fa-flip-horizontal mr-2"></i>11 colonnes',
                    template: '<div class="col-11">',
                },
                'col-sm-11': {},
                'col-md-11': {},
                'col-lg-11': {},
                'col-xl-11': {},

                'col': {
                    label: '<i class="fa fa-list fa-flip-horizontal mr-2"></i>Colonnes auto',
                    template: '<div class="col">',
                },

                'w-100': {
                    label: '<i class="fa fa-list fa-flip-horizontal mr-2"></i>Clear col',
                    template: '<div class="w-100">',
                },
                'col-sm-12': {},
                'col-md-12': {},
                'col-lg-12': {},
                'col-xl-12': {}
            },
            help: "https://getbootstrap.com/docs/4.3/layout/grid/#setting-one-column-width"
        },
        'article': {
            label: 'Ajouter un bloc de texte',
            icon: 'fa-align-left',
            template: '<article class="article"></article>',
            action: 'article',
            belongs: ['col.*', 'w-100'],
            declinations: [],
            help: ""
        },
        'image': {
            label: 'Ajouter une image',
            icon: 'fa-image',
            template: '<img class="image img-fluid" src="/images/default.png" />',
            action: 'image',
            belongs: ['col.*', 'w-100'],
            declinations: [],
            help: ""
        },
    };

})(jQuery);
