@extends('modulus::layouts.app.adminlte')

@section('content')

<div class="card mw-list content-list">

    {{-- Header --}}
    <div class="card-header">
        <div class="row justify-content-between">

            <div class="col-sm-6">

                <div class="form-group mt-1 mb-1">
                    @if (isset($url_back))
                        <a href="{{ $url_back }}" class="btn btn-default btn-sm mr-2">
                            <i class="fa fa-reply mr-1"></i>Retour
                        </a>
                    @endif
                    @yield('action-prepend')
                    <div class="btn-group">
                        @if ($is_creatable)
                            <a href="{{ route($route_prefix . '.create', $route_param ?? '') }}" title="Ajoute un nouvel élément" class="btn btn-success btn-sm">
                                <i class="fa fa-plus mr-1" aria-hidden="true"></i>Ajouter
                            </a>
                        @endif
                        @if (!empty($has_menu))
                            <button type="button" class="btn btn-default btn-sm dropdown-toggle" data-toggle="dropdown" aria-expanded="false">
                            <span class="caret"></span>
                            <span class="sr-only">Toggle Dropdown</span>
                            </button>
                            <div class="dropdown-menu" role="menu" x-placement="bottom-start">
                                @if ($is_exportable)
                                    <a class="dropdown-item" href="{{ route($route_prefix . '.export', $route_param ?? '') }}" title="Exporte la totalité des données">
                                        <i class="fa fa-file-excel-o mr-2" aria-hidden="true"></i>Exporter
                                    </a>
                                @endif

                                @role('admin')
                                    @if (!empty($tags_menu) && is_array($tags_menu))
                                        @foreach ($tags_menu as $name => $tag)
                                            <a href="#" class="dropdown-item clipboard-copy" data-clipboard-action="copy" data-clipboard-text="{{ $tag }}" title="{{ $tag }}">
                                                <i class="fa fa-clipboard mr-1" aria-hidden="true"></i>{{ $name }}
                                            </a>
                                        @endforeach
                                    @endif
                                    @if ($is_resetable)
                                        <a class="dropdown-item confirm" href="#" data-href="{{ route($route_prefix . '.position.reset', $route_param ?? '') }}" title="Réinitialise la position des élements">
                                            <i class="fa fa-sort-amount-asc mr-1" aria-hidden="true"></i>RAZ Position
                                        </a>
                                    @endif
                                @endrole

                                @yield('menu-header')
                                @if ($is_truncable)
                                    <div class="dropdown-divider"></div>
                                    <a class="dropdown-item confirm" href="#" data-href="{{ route($route_prefix . '.truncate', $route_param ?? '') }}" title="Supprime la totalité des données de la liste">
                                        <i class="fa fa-times mr-1" aria-hidden="true"></i>Tout supprimer
                                    </a>
                                @endif
                            </div>
                        @endif
                    </div>
                    @yield('action-append')
                </div>
            </div>

            <div class="col-sm-6 pt-1">
                @if ($is_searchable)
                    <div class="form-group my-0">
                        <form id="filter-form" action="{{ url()->current() }}" method="get">
                            <input type="hidden" name="perpage" value="{{ Request::get('perpage') }}">
                            <div class="row justify-content-end">
                                @yield('filters')
                                <div class="col-md-8 col-lg-6 col-xl-4">
                                    {!! \Field::search() !!}
                                </div>
                            </div>
                        </form>
                    </div>
                @endif
            </div>
        </div>
    </div>

    {{-- Content --}}
    <div class="card-body">
        @yield('list')
        @if ($is_paginable)
            <div class="row mt-3">
                {{ $items->appends(Request::capture()->except('page'))->links() }}
            </div>
            <div class="form-inline pull-right mt-2">
                {!! \Field::perpage() !!}
            </div>
        @endif
    </div>

    <div class="card-footer">
        @if ($is_paginable)
            <span class="pull-right text-muted">{{ $items->total() }} {{ $list_name }}{{ $items->total() > 1 ? 's' : '' }}</span>
        @else
            <span class="pull-right text-muted">{{ count($items) }} {{ $list_name }}{{ count($items) > 1 ? 's' : '' }}</span>
        @endif
    </div>

</div>

@endsection
