<?php

use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/





Route::group(
    [
        'middleware' => ['auth', 'auth.active', 'role:admin|client'],
        'prefix'     => '/',
        'namespace'  => 'App',
        'as'         => 'modulus.'
    ], function ($router) {

    // Tracker
    Route::group(['prefix' => 'tracker', 'as' => 'tracker.'], function() {
        Route::get('export', 'DashboardController@export')->name('export');
        Route::get('truncate', 'DashboardController@truncate')->name('truncate');
    });

    // User
    Route::get('user/profil', 'UserController@profil')->name('user.profil');
    Route::post('user/profil', 'UserController@profilPost')->name('user.profil.post');

    Route::group(['prefix' => 'user', 'as' => 'user.', 'middleware' => ['role:admin']], function() {

        Route::group(['middleware' => ['role:admin']], function() {
            Route::get('/', 'UserController@index')->name('index');
            Route::get('create', 'UserController@create')->name('create');
            Route::get('export', 'UserController@export')->name('export');
            Route::get('update/{id}', 'UserController@update')->name('update');
            Route::post('store/{id?}', 'UserController@store')->name('store');
            Route::get('delete/{id}', 'UserController@delete')->name('delete');
            Route::get('send-created-mail/{id}', 'UserController@sendCreatedAccountMail')->name('send-created-mail');
        });

        /* Permissions */
        Route::group(['prefix' => 'permission', 'as' => 'permission.'], function() {
            Route::get('/', 'PermissionController@index')->name('index');
            Route::get('create', 'PermissionController@create')->name('create');
            Route::get('update/{id}', 'PermissionController@update')->name('update');
            Route::post('store/{id?}', 'PermissionController@store')->name('store');
            Route::get('delete/{id}', 'PermissionController@delete')->name('delete');
            Route::post('search', 'PermissionController@search')->name('search');
        });

        /* Role */
        Route::group(['prefix' => 'role', 'as' => 'role.'], function() {
            Route::get('/', 'RoleController@index')->name('index');
            Route::get('create', 'RoleController@create')->name('create');
            Route::get('update/{id}', 'RoleController@update')->name('update');
            Route::post('store/{id?}', 'RoleController@store')->name('store');
            Route::get('delete/{id}', 'RoleController@delete')->name('delete');
            Route::post('search', 'RoleController@search')->name('search');
        });
    });

    // Modules
    Route::group(['prefix' => 'module', 'as' => 'module.'], function() {
        Route::get('/', 'ModuleController@index')->name('index');
        Route::get('install/{name}', 'ModuleController@install')->name('install');
        Route::get('uninstall/{name}', 'ModuleController@uninstall')->name('uninstall');
        Route::get('refresh/{name}', 'ModuleController@refresh')->name('refresh');
        Route::get('update/{name}', 'ModuleController@update')->name('update');
    });

    // File Manager
    Route::get('filemanager', ['as' => 'filemanager', function() {
        return view('back.elfinder.elfinder')->with(['title' => '<i class="fa fa-file mr-1"></i>Gestionnaire de fichiers']);
    }])->middleware(['permission:file-read', 'permission:file-update', 'permission:file-delete']);
});
